/*----------------------------------------------------------------------
TitanBomber - Roberto Prieto
Copyright (C) 2005-2023 MegaStorm Systems
contact@megastormsystems.com - http://www.megastormsystems.com

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

------------------------------------------------------------------------

Hall of Fame class

------------------------------------------------------------------------ */

#include "hof.h"
#include "gamefield.h"
#include "resource.h"

// cHOF class
cHOF *HOF = NULL;

cHOF::cHOF()
{	
    int i,j;

    // Init our data block
    memset(&bqHOF,0,sizeof(DPF_BlockData));
    strcpy(bqHOF.type,"FILE");
    strcpy(bqHOF.name, "HOF.dat");

    // Try to load data
    idDPF = IStuffDPF->Open(HOF_RESOURCE);
    if(idDPF <= 0) return;
    iStatus = IStuffDPF->Search(idDPF,&bqHOF);
    // No data found, loading default values
    if(iStatus != 1)
    {
        for(i = 0; i < 3; i++)
        {
            for(j = 0; j < HOF_SCORES; j++)
            {
                switch(CRM32Pro.Rand()%4)
                {
                case 0:
                    strcpy(scores[i][j].sPlayerName,"Adam");
                    scores[i][j].iPlayer = PLAYER_WATER;
                    break;
                case 1:
                    strcpy(scores[i][j].sPlayerName,"Mike");
                    scores[i][j].iPlayer = PLAYER_FIRE;
                    break;
                case 2:
                    strcpy(scores[i][j].sPlayerName,"John");
                    scores[i][j].iPlayer = PLAYER_EARTH;
                    break;
                case 3:
                    strcpy(scores[i][j].sPlayerName,"Neil");
                    scores[i][j].iPlayer = PLAYER_AIR;
                    break;
                }				
                scores[i][j].iPoints = (1000 * HOF_SCORES) - (j * 1000);
            }
        }				
    }
    // Load data
    else 
    {
        iStatus = IStuffDPF->LoadBlock(idDPF,"FILE","HOF.dat",&bqHOF);
        memcpy(scores,bqHOF.data,sizeof(scores));
        IStuffDPF->FreeBlock(&bqHOF);
    }
    // Close the DPF
    IStuffDPF->Close(idDPF);
}

cHOF::~cHOF()
{
    // Open dpf
    idDPF = IStuffDPF->Open(HOF_RESOURCE);
    if(idDPF <= 0) return;

    // If the data exists, we remove it
    strcpy(bqHOF.type,"FILE");
    strcpy(bqHOF.name, "HOF.dat");
    iStatus = IStuffDPF->Search(idDPF,&bqHOF);
    if(iStatus == 1) IStuffDPF->Delete(idDPF,&bqHOF);
    // Save data
    bqHOF.size = sizeof(scores);
    bqHOF.data = (char*)scores;
    iStatus = IStuffDPF->AddData(idDPF,&bqHOF);

    // Close dpf
    IStuffDPF->Close(idDPF);

    // Compact the dpf
    IStuffDPF->Compact(HOF_RESOURCE);
}


void cHOF::Store(sPlayers *players, int gametype)
{
    int iP,iS;

    if(players == NULL) return;
    // Adjust the gametype to match our array
    if((gametype < GAMETYPE_LEAGUE) || (gametype > GAMETYPE_RANDOM)) return;
    gametype--; 

    // Get tplayers points and check if they should be in HOF
    for(iP = 0; iP < 4; iP++)
    {
        // Is there a player?
        if(strcmp(players[iP].name,""))		
        {
            for(iS = 0; iS < HOF_SCORES; iS++)
            {
                if(players[iP].points > scores[gametype][iS].iPoints) { Insert(&players[iP],gametype,iS); break; }
            }
        }
    }
}

void cHOF::Insert(sPlayers *player, int gametype, int pos)
{
    int i;

    // Before to insert the new player, we shift one position from the end to post
    for(i = HOF_SCORES - 1; i > pos; i--)
    {
        scores[gametype][i].iPoints = scores[gametype][i - 1].iPoints;
        scores[gametype][i].iPlayer = scores[gametype][i - 1].iPlayer;
        strcpy(scores[gametype][i].sPlayerName, scores[gametype][i - 1].sPlayerName);
    }

    // Insert the new player
    scores[gametype][pos].iPoints = player->points;
    scores[gametype][pos].iPlayer = player->id;
    strcpy(scores[gametype][pos].sPlayerName, player->name);
}

// Render current HOF (position + name + elemental + points)
#define SCORES_X        80
#define SCORES_Y       210
#define SCORES_X_SPACE 245
#define SCORES_Y_SPACE  35
void cHOF::Render()
{
    int  i,iGT,iPos,iX,iLX,iY,iMaxPos,iMaxName,iMaxPoints;
    CRM32Pro_CFont *fPlayer = NULL;
    char sTmp[8];

    // Initial position
    iX = SCORES_X; iY = SCORES_Y;

    // Get the maximum "position text" size used for aligment purposes
    iMaxPos = 0;
    for(iPos = 0; iPos < HOF_SCORES; iPos++)
    {
        _itoa(iPos + 1,sTmp,10);
        i = FontStore->Score->StringWidth(sTmp);
        if(i > iMaxPos) iMaxPos = i;
    }

    // Loop through the game types
    for(iGT = 0; iGT < 3; iGT++)
    {
        // Get the maximum "name text" and "points text" size used for aligment purposes
        // Assumes all custom fonts' players have the same size
        iMaxName = iMaxPoints = 0;
        for(iPos = 0; iPos < HOF_SCORES; iPos++)
        {
            i = FontStore->Water->StringWidth(scores[iGT][iPos].sPlayerName);
            if(i > iMaxName) iMaxName = i;
            _itoa(scores[iGT][iPos].iPoints,sTmp,10);
            i = FontStore->Water->StringWidth(sTmp);
            if(i > iMaxPoints) iMaxPoints = i;
        }

        // Loop through all the scores rendering them aligned
        for(iPos = 0; iPos < HOF_SCORES; iPos++)
        {
            iLX = iX;

            // Position (from 1 to HOF_SCORES)
            _itoa(iPos + 1,sTmp,10);
            FontStore->Score->PutString(CRM32Pro.screen,iLX - (FontStore->Score->StringWidth(sTmp) - iMaxPos),iY,sTmp);
            iLX = iLX + iMaxPos + 10;

            // Player name using the right font
            switch(scores[iGT][iPos].iPlayer)
            {
            case PLAYER_WATER: 
                fPlayer = FontStore->Water;
                break;
            case PLAYER_FIRE: 
                fPlayer = FontStore->Fire;
                break;
            case PLAYER_EARTH: 
                fPlayer = FontStore->Earth;
                break;
            case PLAYER_AIR: 
                fPlayer = FontStore->Air;
                break;
            default:
                fPlayer = FontStore->Info;
                break;
            }
            // Player name
            fPlayer->PutString(CRM32Pro.screen,iLX - (fPlayer->StringWidth(scores[iGT][iPos].sPlayerName) - iMaxName),iY,scores[iGT][iPos].sPlayerName);
            iLX = iLX + iMaxName + 15;

            // Player points
            _itoa(scores[iGT][iPos].iPoints,sTmp,10);
            fPlayer->PutString(CRM32Pro.screen,iLX - (fPlayer->StringWidth(sTmp) - iMaxPoints),iY,sTmp);

            iY = iY + SCORES_Y_SPACE;
        }
        // Change the position for the next scores
        iX = iX + SCORES_X_SPACE;
        iY = SCORES_Y;
    }
}