/*----------------------------------------------------------------------
TitanBomber - Roberto Prieto
Copyright (C) 2005-2023 MegaStorm Systems
contact@megastormsystems.com - http://www.megastormsystems.com

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

------------------------------------------------------------------------

object class

------------------------------------------------------------------------ */

#include "objects.h"
#include "resource.h"

// ---Clase base de Objetos---
cObjects::cObjects()
{
    ILogSystem.Msg(LOG_LOW,"   [cObjects::cObjects()] This isnt a valid constructor.\n");
}
cObjects::cObjects(char *sprname, sPlayers *myself,cGameField *gfield)
{	
    int iTmpX,iTmpY;
    char sTmp[32];

    // Inicializa variables
    sprObject = new CRM32Pro_CSprite();	
    iDirection = DIR_NORMAL; 
    iLifes = 1;
    iSpeed = START_SPEED; 
    iXa = 0; iYa = 0;
    iInvul = 0;

    // Cargamos sprite del objeto
    strcpy(sTmp,sprname);
    strcat(sTmp,"_player");
    sprObject->Load(GFX_RESOURCE,sTmp);
    // Sprites de tamao 50x45, por si el tamao varia, calculamos
    // auto correcciones del objeto y bombas.
    // El tamao por tanto, no afecta a ningun otro calculo.
    sprObject->GetOffset(&iTmpX,&iTmpY);
    iXFix = DELTA_X - iTmpX;
    iYFix = DELTA_Y - iTmpY;
    iID = myself->id;
    // Desactivamos el autorestore
    sprObject->AutoRestore(0);	
    sprObject->SelectAnim(SPRSTATE_NORMAL); 
    // Hook entre el objeto y su campo de juego
    GameField = gfield;
    // Check if we are the server on a network game to protect the access to the player position
    if(GameField->GetMode() == GAMEMODE_SERVER) mutPos = SDL_CreateMutex();
    else mutPos = NULL;
    ILogSystem.Msg(LOG_HIGH,"   [cObjects::cObjects()] Object created: sprite '%s' with ID '%d'.\n",sTmp,iID);
}

cObjects::~cObjects()
{
    if(sprObject != NULL) delete sprObject;
    sprObject = NULL;
    iDirection = 0; iSpeed = 0; 
    iXa = 0; iYa = 0;
    iXFix = iYFix = 0;
    iLifes = 0;
    iInvul = 0;
    GameField = NULL;
    if(mutPos) SDL_DestroyMutex(mutPos);
    mutPos = NULL;
    ILogSystem.Msg(LOG_HIGH,"   [cObjects::~cObjects] Object ID '%d' deleted.\n",iID);
}

Uint8 cObjects::Alive()
{
    return iLifes;
}

int cObjects::Abs2RelX(int iX)
{
    int iTmp;
    iTmp = (iX - ORIGIN_X) / DELTA_X;    
    if(((iX - ORIGIN_X) % DELTA_X) > DELTA_X / 2) iTmp++;
    return iTmp;
}

int cObjects::Abs2RelY(int iY)
{
    int iTmp;
    iTmp = (iY - ORIGIN_Y) / DELTA_Y;    
    if(((iY - ORIGIN_Y) % DELTA_Y) > DELTA_Y / 2) iTmp++;
    return iTmp;
}

void cObjects::Render()
{
    if(sprObject != NULL)
    {		
        // When object is invulnerable, render it with alpha 128
        if(iInvul != 0) sprObject->SetAlpha(128);
        else sprObject->SetAlpha(255);
        sprObject->Draw();
    }
}

// Try to move Y axis on iDir at iSp
void cObjects::TryMoveY(int iDir,int iSp)
{
    Sint32 iTmpX,iTmpY,iAux;
    iTmpX = iXa; 
    iTmpY = iYa; 

    // Convert Abs->Rel position
    int iXr = Abs2RelX(iTmpX);
    int iYr = Abs2RelY(iTmpY);

    // If no iSp is given, we use our speed
    if(iSp == 0) iSp = iSpeed;

    // Request DOWN movement
    if(iDir > 0)
    {		
        // Normalize
        iDir = 1;

        // Check map bounds
        if(iYr == 10)
        {
            // Max Y limit at 533 (Y cells * 45 + 83 = MAP_HEIGHT-1 * DELTA_Y + ORIGIN_Y)
            if((iTmpY + (iSp * iDir)) >= (MAP_HEIGHT - 1) * DELTA_Y + ORIGIN_Y) 
            { 
                iTmpY = (MAP_HEIGHT - 1) * DELTA_Y + ORIGIN_Y; 
                iDir = 0; 
            }
        }
        // Only allow the movement through the walkables + [fire]
        else
        {
            if(GameField->GetStatus(iXr, iYr + 1) <= STATUS_NOWALKABLE) 
            {
                if((iTmpY + (iSp * iDir)) >= iYr * DELTA_Y + ORIGIN_Y) 
                { 
                    iTmpY = iYr * DELTA_Y + ORIGIN_Y; 
                    iDir = 0; 
                }
            }
        }
    }
    // Request UP movement
    else
    {
        // Normalize
        iDir = -1;

        // Check map bounds
        if(iYr == 0)
        {
            // Min Y limit at 83 (0 cell * 45 + 83 = 0 * DELTA_Y + ORIGIN_Y)
            if((iTmpY + (iSp * iDir)) <= ORIGIN_Y) 
            { 
                iTmpY = ORIGIN_Y; 
                iDir = 0;
            }
        }
        // Only allow the movement through the walkables + [fire]
        else
        {
            if(GameField->GetStatus(iXr,iYr - 1) <= STATUS_NOWALKABLE) 
            {
                if((iTmpY + (iSp * iDir)) <= iYr * DELTA_Y + ORIGIN_Y) 
                { 
                    iTmpY = iYr * DELTA_Y + ORIGIN_Y; 
                    iDir = 0; 
                }
            }
        }
    }

    // "Smooth" movement
    iAux = (iTmpX - ORIGIN_X) % DELTA_X;
    if((iAux > 0) && (iDir != 0)) 
    {   
        if(GameField->GetStatus(iXr, iYr + iDir) >= STATUS_WALKABLE)
        {
            int iAdj = iSp;
            if(iAux < DELTA_X / 2)
            {
                if(iSp > iAux) iAdj = iAux;				
                /*if(iSp != iAdj)
                {
                ILogSystem.Msg(LOG_NORMAL,"MoveY() - iTmpX %d - iAux %d - iDir %d -",iTmpX, iAux,iDir);
                ILogSystem.Msg(LOG_NORMAL,"Try left %d.. %d\n",iSp,iAdj);
                }*/
                TryMoveX(-1,iAdj); 
                return;
            }
            else //if(iAux > DELTA_X / 2)
            {
                if(iSp > (DELTA_X - iAux)) iAdj = DELTA_X - iAux;
                /*if(iSp != iAdj) 
                {
                ILogSystem.Msg(LOG_NORMAL,"MoveY() - iTmpX %d - iAux %d - iDir %d -",iTmpX, iAux,iDir);
                ILogSystem.Msg(LOG_NORMAL,"Try right %d..%d\n",iSp,iAdj);
                }*/
                TryMoveX(1,iAdj);  
                return;
            }
        }
    }

    // Modify the position
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_LockMutex(mutPos);
    iXa = iTmpX;
    iYa = iTmpY + (iSp * iDir);
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_UnlockMutex(mutPos);
}

// Try to move X axis on iDir at iSp
void cObjects::TryMoveX(int iDir,int iSp)
{
    Sint32 iTmpX,iTmpY,iAux;
    iTmpX = iXa; 
    iTmpY = iYa; 

    // Abs->Rel position
    int iXr = Abs2RelX(iTmpX);
    int iYr = Abs2RelY(iTmpY);

    // If no iSp is given, we use our speed
    if(iSp == 0) iSp = iSpeed;

    // Request RIGHT movement
    if(iDir > 0)
    {
        // Normalize
        iDir = 1;

        // Check map bounds
        if(iXr == 14)
        {
            // Max X limit at x=725 (X cells * 50 + 25) = MAP_WIDTH-1 * DELTA_X + ORIGIN_X
            if((iTmpX + (iSp * iDir)) >= (MAP_WIDTH - 1) * DELTA_X + ORIGIN_X) 
            { 
                iTmpX = (MAP_WIDTH - 1) * DELTA_X + ORIGIN_X; 
                iDir = 0; 
            }
        }
        // Only allow the movement through the walkables + [fire]
        else
        {
            if(GameField->GetStatus(iXr + 1, iYr) <= STATUS_NOWALKABLE) 
            {
                if((iTmpX + (iSp * iDir)) >= iXr * DELTA_X + ORIGIN_X) 
                { 
                    iTmpX = iXr * DELTA_X + ORIGIN_X; 
                    iDir = 0; 
                }
            }
        }
    }
    // Request LEFT movement
    else 
    {
        // Normalize
        iDir= - 1;

        // Check map bounds
        if(iXr == 0)
        {
            // Max X limt at x=25 (0 cell * 50 + 25) = 0 * DELTA_X + ORIGIN_X
            if((iTmpX + (iSp * iDir)) <= ORIGIN_X) 
            { 
                iTmpX = ORIGIN_X; 
                iDir = 0; 
            }
        }
        // Only allow the movement through the walkables + [fire]
        else
        {
            if(GameField->GetStatus(iXr - 1, iYr) <= STATUS_NOWALKABLE) 
            {
                if((iTmpX + (iSp * iDir)) <= iXr * DELTA_X + ORIGIN_X) 
                { 
                    iTmpX = iXr * DELTA_X + ORIGIN_X; 
                    iDir = 0; 
                }
            }
        }
    }

    // "Smooth" movement
    iAux = (iTmpY - ORIGIN_Y) % DELTA_Y;
    if((iAux > 0))// && (iDir != 0)) 
    {   
        if(GameField->GetStatus(iXr + iDir, iYr) >= STATUS_WALKABLE)
        {
            int iAdj = iSp;
            if(iAux < DELTA_Y / 2)
            {
                if(iSp > iAux) iAdj = iAux;				
                /*if(iSp != iAdj)
                {
                ILogSystem.Msg(LOG_NORMAL,"MoveX() - iTmpY %d - iAux %d - iDir %d -",iTmpY, iAux,iDir);
                ILogSystem.Msg(LOG_NORMAL,"Try up %d.. %d\n",iSp,iAdj);
                }*/
                TryMoveY(-1,iAdj); 
                return;
            }
            else //if(iAux > DELTA_Y / 2)
            {
                if(iSp > (DELTA_Y - iAux)) iAdj = DELTA_Y - iAux;
                /*if(iSp != iAdj) 
                {
                ILogSystem.Msg(LOG_NORMAL,"MoveX() - iTmpY %d - iAux %d - iDir %d -",iTmpY, iAux,iDir);
                ILogSystem.Msg(LOG_NORMAL,"Try down %d..%d\n",iSp,iAdj);
                }*/
                TryMoveY(1,iAdj);  
                return;
            }
        }
    }

    // Modify the position
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_LockMutex(mutPos);
    iXa = iTmpX+(iSp*iDir);
    iYa = iTmpY;
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_UnlockMutex(mutPos);
}

void cObjects::Debug()
{
    char sDebug[15];// Debug string 
    char sTmp[4];   // Auxiliar string
    int iMul;

    strcpy(sDebug,"");
    strcpy(sTmp,"");

    // Dependiendo de que player sea, colocamos sus datos
    switch(iID)
    {
    case PLAYER_WATER:
        iMul = 0;
        break;
    case PLAYER_FIRE:
        iMul = 1;
        break;
    case PLAYER_EARTH:
        iMul = 2;
        break;
    case PLAYER_AIR:
        iMul = 3;
        break;
    }

    _itoa(iXa,sTmp,10);
    strcpy(sDebug,"( ");
    strcat(sDebug,sTmp);
    strcat(sDebug," - ");
    _itoa(iYa,sTmp,10);
    strcat(sDebug,sTmp);
    strcat(sDebug," )");
    FontStore->Debug->PutString(CRM32Pro.screen,60+(iMul*198),57,sDebug);
    _itoa(Abs2RelX(iXa),sTmp,10);
    strcpy(sDebug,"( ");
    strcat(sDebug,sTmp);
    strcat(sDebug," - ");
    _itoa(Abs2RelY(iYa),sTmp,10);
    strcat(sDebug,sTmp);
    strcat(sDebug," )");
    FontStore->Debug->PutString(CRM32Pro.screen,136+(iMul*198),57,sDebug);
}

// Check if I am burning!
// Return: PLAYER_ALIVE or PLAYER_EARTH|PLAYER_AIR|PLAYER_FIRE|PLAYER_WATER that killed us!
Uint8 cObjects::BurnReach()
{
    Uint8 ret;

    // Get the status
    ret = GameField->GetStatus(Abs2RelX(iXa),Abs2RelY(iYa));

    // Detect a player killed
    if((ret >= STATUS_BURN_P1) && (iInvul == 0)) 
    { 	 	
        // Sound and return
        ISoundFX->SoundPlay(-1,hndS_death,0); 
        return ret; 
    }
    else return PLAYER_ALIVE;
}

// Return X position on the map
// Protect it with a mutex as it is accessed by the server thread. In this class, any modification of iXa must be protected
int cObjects::GetXr()
{
    int ret = -1;
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_LockMutex(mutPos);
    ret = Abs2RelX(iXa);
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_UnlockMutex(mutPos);
    return ret;
}

// Return Y position on the map
// Protect it with a mutex as it is accessed by the server thread. In this class, any modification of iYa must be protected
int cObjects::GetYr()
{
    int ret = -1;
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_LockMutex(mutPos);
    ret = Abs2RelY(iYa);
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_UnlockMutex(mutPos);
    return ret;
}


// ---Derivated class cPlayer---
// Constructors
cPlayer::cPlayer()
{
    ILogSystem.Msg(LOG_LOW,"   [cPlayer::cPlayer()] This isnt a valid constructor.\n");
}
cPlayer::cPlayer(char *sprname, sPlayers *myself,cGameField *gfield) : cObjects(sprname,myself,gfield)
{	
    char sTmp[32];

    // First, call the the cObjects constructors and then, init. vars
    int iTmpX,iTmpY,i;
    sprBombMaster = new CRM32Pro_CSprite();	
    sprExplMaster = new CRM32Pro_CSprite();	
    iPoints = iDirDelay = 0;
    iBombDelay = TIME_BOMB;
    iBombPow = START_BOMBPOW; iBombStore = START_BOMBS;
    iXBombFix = iYBombFix = 0;	
    iDeathStatus = iWhoKilledMe = 0;

    // Players have 3 lifes. Add one if we have bonus (we won the last match!)
    iLifes = START_LIFES; 
    if(myself->bonus) { iLifes++; myself->bonus = 0; }

    // Structs init.
    for(i = 0; i < 6; i++) tKeyboard[i] = 0;
    for(i = 0; i < MAX_BOMBS; i++)
    {
        tBombs[i].iBangTime = -1;
        tBombs[i].iExplosionTime = 0;
        tBombs[i].sprBomb = new CRM32Pro_CSprite;		
        tBombs[i].sprExpl = new CRM32Pro_CSprite;	
        tBombs[i].iDown = tBombs[i].iUp = tBombs[i].iLeft = tBombs[i].iRight = 0;
        tBombs[i].iTmpX = tBombs[i].iTmpY = -1;
        tBombs[i].bDrawExplosion = 0;
        tBombs[i].iPower = 0;
    }

    // Get the name
    strcpy(sName,myself->name);
    // Get our type
    iRemote = myself->remote;	
    // Get the ID
    iID = myself->id;

    // Each player on its map corner
    SetInitPosition();
    // Load master sprite and create the clones
    strcpy(sTmp,sprname);
    strcat(sTmp,"_bomb");
    sprBombMaster->Load(GFX_RESOURCE,sTmp);	
    sprBombMaster->AutoRestore(0);
    for(i = 0; i < MAX_BOMBS; i++) tBombs[i].sprBomb->Clone(sprBombMaster);
    // Load master sprite and create the clones
    strcpy(sTmp,sprname);
    strcat(sTmp,"_explosion");
    sprExplMaster->Load(GFX_RESOURCE,sTmp);	
    sprExplMaster->AutoRestore(0);
    for(i = 0; i < MAX_BOMBS; i++) tBombs[i].sprExpl->Clone(sprExplMaster);
    sprBombMaster->GetOffset(&iTmpX,&iTmpY);
    iXBombFix = DELTA_X-iTmpX;
    iYBombFix = DELTA_Y-iTmpY;

    ILogSystem.Msg(LOG_HIGH,"   [cPlayer::cPlayer()] Player '%s' (ID '%d') created.\n",sName,iID);
}

// Destructor
cPlayer::~cPlayer()
{
    int i;
    for(i=0;i<MAX_BOMBS;i++)
    {
        tBombs[i].iBangTime = -1;
        tBombs[i].iExplosionTime = 0;
        if(tBombs[i].sprBomb != NULL) delete tBombs[i].sprBomb;
        if(tBombs[i].sprExpl != NULL) delete tBombs[i].sprExpl;
        tBombs[i].iDown = tBombs[i].iUp = tBombs[i].iLeft = tBombs[i].iRight = 0;
        tBombs[i].iTmpX = tBombs[i].iTmpY = -1;
        tBombs[i].bDrawExplosion = 0;
        tBombs[i].iPower = 0;
    }
    if(sprBombMaster != NULL) delete sprBombMaster;
    if(sprExplMaster != NULL) delete sprExplMaster;
    iBombPow = 1; iBombStore = 1;
    iXBombFix = iYBombFix = 0;
    iBombDelay = iDirDelay = 0;
    iLifes = 0;
    iPoints = 0;	
    ILogSystem.Msg(LOG_HIGH,"   [cPlayer::~cPlayer()] Player '%s' (ID '%d') deleted.\n",sName,iID);
}


// Event control for local/remote player. Send/Receive the network events.
void cPlayer::Control(int nPlayer)
{	
    int iX=0,iY=0,iBomb=0,done;

    // 1. Local player checks the keyboard if it has lives and it isnt dying
    if( (iRemote == 0) && (iLifes > 0) && (iDeathStatus == 0))
    {
        // Get events from the keyboard (movement and bomb creation)
        if(CRM32Pro.GetKeystate(tKeyboard[KEY_DOWN])) iY = 1;
        if(CRM32Pro.GetKeystate(tKeyboard[KEY_UP])) iY = 255;
        if(CRM32Pro.GetKeystate(tKeyboard[KEY_RIGHT])) iX = 1;
        if(CRM32Pro.GetKeystate(tKeyboard[KEY_LEFT])) iX = 255;
        if(CRM32Pro.GetKeystate(tKeyboard[KEY_BOMB])) 
        {
            if(BombCreatePreValidate()) iBomb = 1;
        }		

        // If this is a network game. Send my actions to the server to be validated.
        if((iBomb!=0) || (iX!=0) || (iY!=0))
        {
            if(GameField->GetMode() != GAMEMODE_LOCAL)
            {		
                TBDN_Event.iCtrlID = iID;
                TBDN_Event.event.move_x = iX;
                TBDN_Event.event.move_y = iY;
                TBDN_Event.event.Xr = GetXr();
                TBDN_Event.event.Yr = GetYr();
                TBDN_Event.event.bomb = iBomb;
                TBDN_Event.event.item = ITEM_NULL;	// No item on this event
                TBDN_Event.event.points = 0;		// No points on this event
                TBDN_Event.event.death = 0;			// No death event
                INetwork->SendData((char *)&TBDN_Event,sizeof(TBDN_Event),iBomb); // Bypass server callback with no bomb event
            }
        }
    }

    // 2. Only for network games
    // We allow to execute this code although we are dead or dying just to keep processing events
    if(GameField->GetMode() != GAMEMODE_LOCAL)
    {
        // Reset event values
        iX = 0;
        iY = 0;
        iBomb = 0;

        // Get an event from the queue. 
        // With item, points or death events, get another event (movement?)
        done = 0;
        while(!done)
        {
            done = 1;
            pTBDN_Event = GameField->GetEventForPlayer(nPlayer);

            if(pTBDN_Event != NULL)
            {
                // Get event control values
                iX = pTBDN_Event->event.move_x;
                iY = pTBDN_Event->event.move_y;
                iBomb = pTBDN_Event->event.bomb;	
                // Get item validations
                if(pTBDN_Event->event.item != ITEM_NULL) { ItemCatch(pTBDN_Event->event.item,pTBDN_Event->event.Xr,pTBDN_Event->event.Yr); done = 0; }
                // Get points validations
                if(pTBDN_Event->event.points != 0) { AddPoints(pTBDN_Event->event.points,1,pTBDN_Event->event.Xr,pTBDN_Event->event.Yr); done = 0; }
                // Get death validation
                if(pTBDN_Event->event.death != 0) { iWhoKilledMe = pTBDN_Event->event.death; iDeathStatus = 1; done = 0; }
#ifdef DEBUG_NETWORK_FULL
                ILogSystem.Msg(LOG_NORMAL,"   [Player::Control] - Rel(%d,%d) Points(%d) Bomb(%d) Item(%d) Death(%d) for '%d'\n",
                    pTBDN_Event->event.Xr,pTBDN_Event->event.Yr,pTBDN_Event->event.points,iBomb,pTBDN_Event->event.item,pTBDN_Event->event.death,iID);
#endif
            }
        }
    }   
    // 3. Are we dead?
    if(iLifes == 0) return;

    // 4. Am I dying?
    if(iDeathStatus != 0) return;

    // 5. Execute current event
    ControlExecute(iX,iY,iBomb);
}

// Execute the actions of Control
void cPlayer::ControlExecute(int iX,int iY,int iBomb)
{
    int iTmp;

    if(iY == 1)
    {
        TryMoveY(1);
        if(iDirection != DIR_DOWN) sprObject->SelectAnim(SPRSTATE_DOWN);
        iDirection = DIR_DOWN;
        iDirDelay = ITimeSystem->GetTime();
    }
    else if(iY == 255)
    {		
        TryMoveY(-1);
        if(iDirection != DIR_UP) sprObject->SelectAnim(SPRSTATE_UP);
        iDirection = DIR_UP;
        iDirDelay = ITimeSystem->GetTime();
    }
    else if(iX == 1)
    {
        TryMoveX(1);
        if(iDirection != DIR_RIGHT) sprObject->SelectAnim(SPRSTATE_RIGHT); 
        iDirection = DIR_RIGHT;
        iDirDelay = ITimeSystem->GetTime();
    }
    else if(iX == 255)
    {		  
        TryMoveX(-1);
        if(iDirection != DIR_LEFT) sprObject->SelectAnim(SPRSTATE_LEFT);
        iDirection = DIR_LEFT;
        iDirDelay = ITimeSystem->GetTime();
    }
    else
    { 		
        iTmp = ITimeSystem->GetTime();
        if((iDirection != 0) && (iTmp > iDirDelay + TIME_DIR))
        {
            sprObject->SelectAnim(SPRSTATE_NORMAL); 
            iDirection = iDirDelay = 0;
        }		  
    }
    if(iBomb == 1) BombCreate();
}

// Display player info
void cPlayer::InfoPlayer()
{
    int iMul,iTmp;
    char sDebug[6]; // Debug string 
    char sTmp[6];   // Auxiliar string
    strcpy(sDebug,"");
    strcpy(sTmp,"");

    switch(iID)
    {
    case PLAYER_WATER:
        iMul = 0;
        break;
    case PLAYER_FIRE:
        iMul = 1;
        break;
    case PLAYER_EARTH:
        iMul = 2;
        break;
    case PLAYER_AIR:
        iMul = 3;
        break;
    }
    // Render player name (5 chars)
    // 4 0's are 72p wide. Auto center
    FontStore->Info->PutString(CRM32Pro.screen,OINFO_X+(OINFO_UNIT*iMul)+SCORE_X+9+((72-FontStore->Info->StringWidth(sName))/2),OINFO_Y+3,sName);

    // Render points (5 chars)
    _itoa(iPoints,sTmp,10);
    iTmp = (int)strlen(sTmp);
    iTmp = iTmp - 5; if(iTmp < 0) iTmp = iTmp * (-1);
    while(iTmp != 0) 
    {
        strcat(sDebug,"0");
        iTmp--;
    }
    strcat(sDebug,sTmp);    	
    // 5 0's are 90p wide. Right justify
    FontStore->Score->PutString(CRM32Pro.screen,OINFO_X+(OINFO_UNIT*iMul)+SCORE_X+(90-FontStore->Score->StringWidth(sDebug)),OINFO_Y+SCORE_Y,sDebug);

    // Render lifes, bombs, explosion and speed
    _itoa(iLifes,sDebug,10);
    FontStore->Info->PutString(CRM32Pro.screen,OINFO_X+(OINFO_UNIT*iMul)+FONT1_X,OINFO_Y+FONT1_Y,sDebug);
    _itoa(iBombStore,sDebug,10);
    FontStore->Info->PutString(CRM32Pro.screen,OINFO_X+(OINFO_UNIT*iMul)+FONT1_X,OINFO_Y+FONT2_Y,sDebug);
    _itoa(iBombPow,sDebug,10);
    FontStore->Info->PutString(CRM32Pro.screen,OINFO_X+(OINFO_UNIT*iMul)+FONT2_X,OINFO_Y+FONT1_Y,sDebug);
    _itoa(iSpeed,sDebug,10);
    FontStore->Info->PutString(CRM32Pro.screen,OINFO_X+(OINFO_UNIT*iMul)+FONT2_X,OINFO_Y+FONT2_Y,sDebug);
}

// Object logic execution
Uint8 cObjects::Execute()
{    
    // No more lifes!
    if(iLifes == 0) return PLAYER_DEATH;

    // Check if we are burning!
    return BurnReach();
}

// Player logic execution
Uint8 cPlayer::Execute()
{    
    Uint8 iTmp,iRet = PLAYER_ALIVE;

    // Set our absolute position (on screen) plus the auto adjustment
    sprObject->SetPosition(iXa + iXFix,iYa + iYFix,1);

    // Execute the bombs. Although the player could be death, it still can have bombs..
    BombExecute();

    // We are dead and End Of the Match!
    if(iLifes == 0) return PLAYER_EOMATCH;

    // Small Death State Machine:
    // Al morir, no nos movemos, no acciones y no se puede morir mas que una vez.
    // Reproduce animacion de muerte, al terminar, nos quitamos una vida.
    // Si tenemos mas vidas, reaparecemos en nuestra posicion original con un tiempo de inmunidad. 
    // Si no nos quedan, morimos.Comprobamos si nos matamos nosotros mismos
    switch(iDeathStatus)
    {
        // Still alive and no dead-query issued?
    case 0:	
        // Check if we are on invul. status
        if(iInvul != 0)
        {
            if(iInvul + TIME_INVUL < ITimeSystem->GetTime()) 
            {
                iInvul = 0;
                ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' is again vulnerable!\n",sName);
            }
            iTmp = PLAYER_ALIVE;
        }
        // Object logic execution (check if we are burning!)
        else iTmp = cObjects::Execute();

        // Is a local game?
        if(GameField->GetMode() == GAMEMODE_LOCAL)
        {				
            if(iTmp & PLAYER_ALIVE) { iWhoKilledMe = 0; }
            else { iWhoKilledMe = iTmp; iDeathStatus = 1; }
            // Get items
            ItemCatch(0,GetXr(),GetYr());
        }
        // Network game
        else
        {
            // Each "local" player perform these queries, no other "remote" players, although they are the same player :)
            // Remove possible wrong and/or duplicated events
            if(!iRemote) 
            {
                // Query a death check to the server. Processed on Control()
                if(iTmp & PLAYER_ALIVE) { iWhoKilledMe = 0; }
                else DeathCheckQuery(iTmp);
                // Query an item catch. Processed on Control()
                ItemCatchQuery();
            }
        }						
        break;

        // We detected that our position is burning! Start the dead process
    case 1:
        ILogSystem.Msg(LOG_HIGH,"  -> Dead SM: going to state 2\n");
        sprObject->SelectAnim(SPRSTATE_NORMAL+1);
        iDeathStatus++; // Next state
        break;

        // Dying! wait until the death animation 1 finished 1/2
    case 2:
        // Synchronize with the end of the first death animation
        if(sprObject->IsFinished()) 
        {
            ILogSystem.Msg(LOG_HIGH,"  -> Dead FSM: going to state 3\n");
            sprObject->SelectAnim(SPRSTATE_NORMAL+2);
            iDeathStatus++; // Next state
        }
        break;

        // Dying! wait until the death animation 2 finished 2/2
    case 3:
        // Synchronize with the end of the second death animation
        if(sprObject->IsFinished()) 
        {
            ILogSystem.Msg(LOG_HIGH,"  -> Dead FSM: going to state 4\n");
            iDeathStatus++; // Next state
        }
        break;

        // Now, we are death. Assign points and set on the invulnerability
    case 4:
        SetInitPosition();
        iLifes--;
        iRet = PLAYER_KILLED; // At least, we've lost one life...

        // Was it an autokill?
        if(iWhoKilledMe & iID) 
        { 
            ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' was killed by itself!\n",sName); 
        } 
        // Print the killer(s) 
        else ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' was killed by '%s'!\n",sName,GameField->GetPlayerName(iWhoKilledMe)); 

        // Are we running out of lifes?
        if(iLifes == 0) 
        { 
            ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' has dead!\n",sName); 
            // Return the killer(s) and set that Im death! plus KILLED or AUTOKILL
            iRet += PLAYER_DEATH|iWhoKilledMe; 
        }  
        // No, we still have lifes
        else 
        {
            // Reset all vars
            iDeathStatus = 0;
            sprObject->SelectAnim(SPRSTATE_NORMAL); 
            // Start the invulnerability time
            iInvul = ITimeSystem->GetTime();
            ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' has %d seconds of invulnerability!\n",sName,TIME_INVUL/1000); 
            iRet += iWhoKilledMe;
        }			
        break;
        // Special "survival" status set by Survive() called by GameField->Execute()
        // We do nothing!
    case 5:
        break;
    }    
    // Return player alive, killed or death (who was the killer)
    return iRet;
}

// Special player status used when the player survived to TIME_EOM
void cPlayer::Survive()
{
    iDeathStatus = 5;
    sprObject->SelectAnim(SPRSTATE_NORMAL);
    iInvul = 1;
}

// Get player's point
int cPlayer::GetPoints()
{
    return iPoints;
}

// Pick up items. Used for local games or for network games after the query
// In network games, after validates the query, it will call this member with the item
void cPlayer::ItemCatch(Uint8 iItem, int iCX, int iCY)
{
    float fSpeed;

    if(iItem == 0) iItem = GameField->GetItem(iCX,iCY);

    // Check if there is an item
    switch(iItem)
    {
    case ITEM_BK_BOMB:
        if(iBombStore < MAX_BOMBS) iBombStore++;            
        break;
    case ITEM_BK_FIREUP:
        if(iBombPow < MAX_BOMBPOW) iBombPow++;
        break;
    case ITEM_BK_SPEEDUP:
        if(iSpeed < MAX_SPEED) iSpeed++;
        fSpeed = (float)iSpeed / (float)4.0;
        sprObject->SetAnimFactor(fSpeed);
        break;
    case ITEM_BK_SPEEDDOWN:
        if(iSpeed > MIN_SPEED) iSpeed--;
        fSpeed = (float)iSpeed / (float)4.0;
        sprObject->SetAnimFactor(fSpeed);
        break;
    default:
        return;                         
    }

    // Try to remove the item
    GameField->BreakablesKill(iCX,iCY);

    // Add the points
    AddPoints(POINTS_ITEM,0,iCX,iCY);

    // Sound
    ISoundFX->SoundPlay(-1,hndS_catchItem,0);
}

// Query to pick up an item. Used by network games
void cPlayer::ItemCatchQuery()
{
    Uint8 iItem;

    // Check if there is an item.
    iItem = GameField->GetItem(GetXr(),GetYr());
    if(iItem != ITEM_NULL)
    {
        // Send a query to get it
        TBDN_Event.iCtrlID = iID;
        TBDN_Event.event.move_x = 0;
        TBDN_Event.event.move_y = 0;
        TBDN_Event.event.Xr = GetXr();
        TBDN_Event.event.Yr = GetYr();
        TBDN_Event.event.bomb = 0;
        TBDN_Event.event.item = iItem;
        TBDN_Event.event.points = 0;
        TBDN_Event.event.death = 0;
        INetwork->SendData((char *)&TBDN_Event,sizeof(TBDN_Event));
        //ILogSystem.Msg(LOG_NORMAL,"   [Player::ItemCatchQuery] - Query sent! (xr%d-yr%d-item%d)\n",TBDN_Event.event.iXr,TBDN_Event.event.iYr,TBDN_Event.event.item);
    }    
}

// Query to request a possible death (to be confirmed by the server). Used by network games
void cPlayer::DeathCheckQuery(Uint8 death)
{
    // Send a query to get it
    TBDN_Event.iCtrlID = iID;
    TBDN_Event.event.move_x = 0;
    TBDN_Event.event.move_y = 0;
    TBDN_Event.event.Xr = GetXr();
    TBDN_Event.event.Yr = GetYr();
    TBDN_Event.event.bomb = 0;
    TBDN_Event.event.item = 0;
    TBDN_Event.event.points = 0;
    TBDN_Event.event.death = death;
    INetwork->SendData((char *)&TBDN_Event,sizeof(TBDN_Event));
    //ILogSystem.Msg(LOG_NORMAL,"   [Player::DeathCheckQuery] - Query sent! (xr%d-yr%d-death%d)\n",TBDN_Event.event.iXr,TBDN_Event.event.iYr,TBDN_Event.event.death);
}

// Assign keyset to this player
void cPlayer::SetCtrlKeys(int iKeySet)
{
    int tKeys[4][5];

    // Keys set
    tKeys[0][0] = SDLK_UP; tKeys[0][1] = SDLK_DOWN;	tKeys[0][2] = SDLK_LEFT; tKeys[0][3] = SDLK_RIGHT; tKeys[0][4] = SDLK_RCTRL;
    tKeys[1][0] = SDLK_a; tKeys[1][1] = SDLK_z;	tKeys[1][2] = SDLK_c; tKeys[1][3] = SDLK_v; tKeys[1][4] = SDLK_LALT;
    tKeys[2][0] = SDLK_KP8; tKeys[2][1] = SDLK_KP2;	tKeys[2][2] = SDLK_KP4; tKeys[2][3] = SDLK_KP6; tKeys[2][4] = SDLK_KP0;
    tKeys[3][0] = SDLK_j; tKeys[3][1] = SDLK_m;	tKeys[3][2] = SDLK_COMMA; tKeys[3][3] = SDLK_PERIOD; tKeys[3][4] = SDLK_RALT;

    // Assign keys to this player
    tKeyboard[KEY_UP] = tKeys[iKeySet][0];
    tKeyboard[KEY_DOWN] = tKeys[iKeySet][1];
    tKeyboard[KEY_LEFT] = tKeys[iKeySet][2];
    tKeyboard[KEY_RIGHT] = tKeys[iKeySet][3];
    tKeyboard[KEY_BOMB] = tKeys[iKeySet][4];
}

// Add points. 
// In network games, the clients do nothing! Only the server when executes each player, generates the points and send them on events
// It bypass the server callback as points are always validated due to only server send them so we assure only one sending point!
void cPlayer::AddPoints(int points, int force, int tx, int ty)
{
    char sValue[16];
    int iX,iY;

    // Server send the points to all players. With force, this is for the client side: add the points!
    if((GameField->GetMode() == GAMEMODE_SERVER) && (force == 0))
    {
        TBDN_Event.iCtrlID = iID;
        TBDN_Event.event.move_x = 0;
        TBDN_Event.event.move_y = 0;
        TBDN_Event.event.Xr = tx;
        TBDN_Event.event.Yr = ty;
        TBDN_Event.event.bomb = 0;
        TBDN_Event.event.item = ITEM_NULL;	// No item on this event
        TBDN_Event.event.points = points;	// The points to be added
        TBDN_Event.event.death = 0;
        INetwork->SendData((char *)&TBDN_Event,sizeof(TBDN_Event),0); // Bypass the server callback function
        ILogSystem.Msg(LOG_NORMAL,"   [Player::AddPoints] - Send event (%d,%d) - Points %d for '%d'\n",tx,ty,points,iID);
        return;
    }

    // Clients do nothing! but with force = 1, we allow to continue as it is a network event
    if((GameField->GetMode() != GAMEMODE_LOCAL) && (force == 0)) return;


    // Detects if we should add points or multiply per the coefficient
    if(points == POINTS_LOCALWIN) 
    {
        iPoints = (iPoints * POINTS_LOCALWIN) / 100;
        ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' has got a local win, final points: '%d'\n",sName,iPoints);
    }
    else if(points == POINTS_VISITORWIN) 
    {
        iPoints = (iPoints * POINTS_VISITORWIN) / 100;
        ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' has got a visitor win, final points: '%d'\n",sName,iPoints);
    }
    else 
    {
        iPoints = iPoints + (points);
        //ILogSystem.Msg(LOG_NORMAL,"  -> Player '%s' gets '%d' points\n",sName,points);

        // Convert the points to a string
        _itoa(points,sValue,10);

        // Get the (x,y) of the points. Centered on the box.
        iX = (ORIGIN_X + (tx * DELTA_X)) + ((DELTA_X - FontStore->GetPlayerFont(iID)->StringWidth(sValue)) / 2);
        iY = (ORIGIN_Y + (ty * DELTA_Y)) + ((DELTA_Y - 20) / 2);

        // Add the floating message
        GameField->AddFloatingMsg(iX,iY,sValue,FontStore->GetPlayerFont(iID),2000);
        //ILogSystem.Msg(LOG_NORMAL," > New floating message (%d,%d): '%s' font '%x'\n",tx,ty,sValue,FontStore->GetPlayerFont(iID));
    }

    // Limits
    if(iPoints < 0) iPoints = 0;
    if(iPoints > 99999) iPoints = 99999;	 
}    

// Initialize the player position
// Protected with a mutex
void cPlayer::SetInitPosition()
{
    int iXr,iYr;

    switch(iID)
    {
    case PLAYER_WATER:
        iXr = 0; iYr = 0; 
        break;
    case PLAYER_FIRE:
        iXr = 14; iYr = 0;
        break;
    case PLAYER_EARTH:
        iXr = 0; iYr = 10;
        break;
    case PLAYER_AIR:
        iXr = 14; iYr = 10;
        break;
    }
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_LockMutex(mutPos);
    iXa = (iXr * DELTA_X) + ORIGIN_X;
    iYa = (iYr * DELTA_Y) + ORIGIN_Y;
    if(GameField->GetMode() == GAMEMODE_SERVER) SDL_UnlockMutex(mutPos);
    sprObject->SetPosition(iXa + iXFix,iYa + iYFix,0);
}

// Prevalidate a bomb creation request
int cPlayer::BombCreatePreValidate()
{
    int i,iNBombs = 0;

    if(GameField->GetStatus(GetXr(), GetYr()) <= STATUS_NOWALKABLE)	return 0;

    // Count how many bombs I have on the map
    for(i = 0; i < MAX_BOMBS; i++)
    {
        if(tBombs[i].iBangTime != -1) iNBombs++;
    }
    if(iNBombs >= iBombStore) return 0;

    // Go ahead to the bomb creation
    return 1;
}

// Create a new bomb
void cPlayer::BombCreate()
{
    int i;	

    // Create a new bomb
    i = 0;
    while(tBombs[i].iBangTime != -1) i++;
    tBombs[i].iBangTime = ITimeSystem->GetTime();
    tBombs[i].sprBomb->SetPosition(GetXr()*DELTA_X+ORIGIN_X+iXBombFix,GetYr()*DELTA_Y+ORIGIN_Y+iYBombFix);
    tBombs[i].iOwner = iID;
    tBombs[i].bDrawExplosion = 0;
    tBombs[i].iPower = iBombPow;
    tBombs[i].iTmpX = -1;
    GameField->SetStatus(GetXr(),GetYr(),STATUS_NOWALKABLE);
    ILogSystem.Msg(LOG_HIGH,"  -> New bomb created on (%d,%d:%d)\n",GetXr(),GetYr(),tBombs[i].iBangTime);

    // Sound
    ISoundFX->SoundPlay(-1,hndS_putBomb,0);
}


// Bomb logic execution
void cPlayer::BombExecute()
{
    int i,k,iTmp,iStatus,iItem=0;
    char type;
    Uint32 iTime;
    iTime = ITimeSystem->GetTime();

    // Loop through all player bombs
    for(i = 0; i < MAX_BOMBS; i++)
    {
        // Bomb position (only first time)
        if(tBombs[i].iTmpX == -1) 
        {
            tBombs[i].iTmpX = (tBombs[i].sprBomb->GetPosX() - ORIGIN_X - iXBombFix) / DELTA_X;
            tBombs[i].iTmpY = (tBombs[i].sprBomb->GetPosY() - ORIGIN_Y - iYBombFix) / DELTA_Y;
        }

        // Active bomb waiting for its explosion
        if(tBombs[i].iBangTime > 0)
        {
            // Next iteration... bomb explosion!
            if(iTime > (Uint32)(tBombs[i].iBangTime + iBombDelay)) tBombs[i].iBangTime=0; 

            // Check if the bomb is under the fire of other bomb...
            if(GameField->GetStatus(tBombs[i].iTmpX,tBombs[i].iTmpY) >= STATUS_BURN_P1)
            {
                tBombs[i].iBangTime = 0;
            }
        }
        // Starting the exploting!
        else if(tBombs[i].iBangTime == 0)
        {	
            // Starting the explosion...
            GameField->SetStatus(tBombs[i].iTmpX,tBombs[i].iTmpY,iID);
            ILogSystem.Msg(LOG_HIGH,"  -> Bomb '%d' exploting on (%d,%d)\n",i,tBombs[i].iTmpX,tBombs[i].iTmpY);
            tBombs[i].iBangTime = -2;
            tBombs[i].iExplosionTime = ITimeSystem->GetTime();
            // Add points
            AddPoints(POINTS_BOMB,0,tBombs[i].iTmpX,tBombs[i].iTmpY);
            // Sound
            switch(iID)
            {
            case PLAYER_WATER:
                ISoundFX->SoundPlay(-1,hndS_waterExplosion,0);
                break;
            case PLAYER_FIRE:
                ISoundFX->SoundPlay(-1,hndS_fireExplosion,0);
                break;
            case PLAYER_EARTH:
                ISoundFX->SoundPlay(-1,hndS_earthExplosion,0);
                break;
            case PLAYER_AIR:
                ISoundFX->SoundPlay(-1,hndS_airExplosion,0);
                break;
            }
        }
        // Exploting bomb durint TIME_FIRE time
        else if(tBombs[i].iBangTime == -2)
        {			
            type = iID;

            // Check for the end of explosion
            if(iTime > (Uint32)(tBombs[i].iExplosionTime + TIME_FIRE)) 
            {
                tBombs[i].iBangTime = -1;
                type = STATUS_WALKABLE;
            }

            // Next draw loop will blit the explosion and avoid to recalculate the bomb explosion limits
            // It only passes the first and last bomb execution.
            // In the first pass, it calculates the explosion limits
            // In the last pass, it removes the explosion without calculate anything
            if((!tBombs[i].bDrawExplosion) || (tBombs[i].iBangTime == -1))
            {
                tBombs[i].bDrawExplosion = 1;

                // Central explosion 
                GameField->SetStatus(tBombs[i].iTmpX,tBombs[i].iTmpY,type);

                // Left explosion. First time calculates the explosion limits. Second (and last) time, restore the status
                if(tBombs[i].iBangTime != -1)
                {
                    iTmp = tBombs[i].iTmpX - tBombs[i].iPower;
                    if(iTmp < 0) iTmp = 0;
                    for(k = tBombs[i].iTmpX; k != iTmp; k--) // Explosion field till a block,item or breakable
                    { 
                        iItem = GameField->GetItem(k-1,tBombs[i].iTmpY);
                        if(iItem == ITEM_BLOCK) break; 
                        if(iItem >= ITEM_BREAKABLE) 
                        { 
                            iStatus = GameField->BreakablesKill(k-1,tBombs[i].iTmpY);
                            if(iStatus == 4) AddPoints(POINTS_BK,0,k-1,tBombs[i].iTmpY); 
                            // It is an item, destroy and add +1 to the explosion
                            else if(iStatus == 6) 
                            { 
                                GameField->SetStatus(k-1,tBombs[i].iTmpY,type);
                                k--;
                                AddPoints(POINTS_ITEM,0,k,tBombs[i].iTmpY); 
                            }
                            break; 
                        }
                        // Set the state
                        GameField->SetStatus(k-1,tBombs[i].iTmpY,type);
                    }
                    // Set the left limit for the BombRender() and for restoring the status on the second and last time
                    tBombs[i].iLeft = k;
                }
                else
                {
                    for(k = tBombs[i].iTmpX; k != tBombs[i].iLeft; k--)
                    {
                        // Set the state
                        GameField->SetStatus(k-1,tBombs[i].iTmpY,type);
                    }
                }			

                // Right explosion. First time calculates the explosion limits. Second (and last) time, restore the status
                if(tBombs[i].iBangTime != -1)
                {
                    iTmp = tBombs[i].iTmpX + tBombs[i].iPower;
                    if(iTmp > 14) iTmp = 14;
                    for(k = tBombs[i].iTmpX; k != iTmp; k++) // Explosion field till a block,item or breakable
                    { 
                        iItem = GameField->GetItem(k+1,tBombs[i].iTmpY);
                        if(iItem == ITEM_BLOCK) break; 
                        if(iItem >= ITEM_BREAKABLE) 
                        { 
                            iStatus = GameField->BreakablesKill(k+1,tBombs[i].iTmpY);
                            if(iStatus == 4) AddPoints(POINTS_BK,0,k+1,tBombs[i].iTmpY); 
                            // It is an item, destroy and add +1 to the explosion
                            else if(iStatus == 6) 
                            { 
                                GameField->SetStatus(k+1,tBombs[i].iTmpY,type);
                                k++;
                                AddPoints(POINTS_ITEM,0,k,tBombs[i].iTmpY); 
                            }
                            break; 
                        }
                        // Set the state
                        GameField->SetStatus(k+1,tBombs[i].iTmpY,type);
                    }
                    // Set the right limit for the BombRender() and for restoring the status on the second and last time
                    tBombs[i].iRight = k;
                }
                else
                {
                    for(k = tBombs[i].iTmpX; k != tBombs[i].iRight; k++)
                    {
                        // Set the state
                        GameField->SetStatus(k+1,tBombs[i].iTmpY,type);
                    }
                }	


                // Up explosion. First time calculates the explosion limits. Second (and last) time, restore the status
                if(tBombs[i].iBangTime != -1)
                {
                    iTmp = tBombs[i].iTmpY - tBombs[i].iPower;
                    if(iTmp < 0) iTmp = 0;
                    for(k = tBombs[i].iTmpY; k != iTmp; k--) // Explosion field till a block,item or breakable
                    { 
                        iItem = GameField->GetItem(tBombs[i].iTmpX,k-1);
                        if(iItem == ITEM_BLOCK) break; 
                        if(iItem >= ITEM_BREAKABLE) 
                        { 
                            iStatus = GameField->BreakablesKill(tBombs[i].iTmpX,k-1);
                            if(iStatus == 4) AddPoints(POINTS_BK,0,tBombs[i].iTmpX,k-1); 
                            // It is an item, destroy and add +1 to the explosion
                            else if(iStatus == 6) 
                            { 
                                GameField->SetStatus(tBombs[i].iTmpX,k-1,type);
                                k--;
                                AddPoints(POINTS_ITEM,0,tBombs[i].iTmpX,k); 
                            }
                            break; 
                        }
                        // Set the state
                        GameField->SetStatus(tBombs[i].iTmpX,k-1,type);
                    }
                    // Set the up limit for the BombRender() and for restoring the status on the second and last time
                    tBombs[i].iUp = k;
                }
                else
                {
                    for(k = tBombs[i].iTmpY; k != tBombs[i].iUp; k--)
                    {
                        // Set the state
                        GameField->SetStatus(tBombs[i].iTmpX,k-1,type);
                    }
                }	


                // Down explosion. First time calculates the explosion limits. Second (and last) time, restore the status
                if(tBombs[i].iBangTime != -1)
                {
                    iTmp = tBombs[i].iTmpY + tBombs[i].iPower;
                    if(iTmp > 10) iTmp = 10;
                    for(k = tBombs[i].iTmpY; k != iTmp; k++) // Explosion field till a block,item or breakable
                    { 
                        iItem = GameField->GetItem(tBombs[i].iTmpX,k+1);
                        if(iItem == ITEM_BLOCK) break; 
                        if(iItem >= ITEM_BREAKABLE) 
                        { 
                            iStatus = GameField->BreakablesKill(tBombs[i].iTmpX,k+1);
                            if(iStatus == 4) AddPoints(POINTS_BK,0,tBombs[i].iTmpX,k+1); 
                            // It is an item, destroy and add +1 to the explosion
                            else if(iStatus == 6) 
                            { 
                                GameField->SetStatus(tBombs[i].iTmpX,k+1,type);
                                k++;
                                AddPoints(POINTS_ITEM,0,tBombs[i].iTmpX,k); 
                            }
                            break; 
                        }
                        // Set the up limit for the BombRender()
                        GameField->SetStatus(tBombs[i].iTmpX,k+1,type);
                    }
                    // Set the down limit for the BombRender() and for restoring the status on the second and last time
                    tBombs[i].iDown = k;         
                }
                else
                {
                    for(k = tBombs[i].iTmpY; k != tBombs[i].iDown; k++)
                    {
                        // Set the state
                        GameField->SetStatus(tBombs[i].iTmpX,k+1,type);
                    }
                }	
            }
        }
    }
}

// Dibujado de bombas
void cPlayer::BombRender()
{
    int i,iTmp;

    // Loop through all player bombs
    for(i = 0; i < MAX_BOMBS; i++)
    {
        // Active bomb waiting for its explosion
        if(tBombs[i].iBangTime > 0)
        {
            tBombs[i].sprBomb->Draw();
        }

        // Exploting bomb!
        else if((tBombs[i].iBangTime == -2) && (tBombs[i].bDrawExplosion))
        {
            // Central explosion 
            tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL);
            tBombs[i].sprExpl->SetPosition(tBombs[i].iTmpX * DELTA_X + ORIGIN_X, tBombs[i].iTmpY * DELTA_Y + ORIGIN_Y);
            tBombs[i].sprExpl->Draw();

            // Left explosion
            if(tBombs[i].iLeft != tBombs[i].iTmpX)
            {
                tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL + 2);
                tBombs[i].sprExpl->SetPosition((tBombs[i].iLeft) * DELTA_X + ORIGIN_X, tBombs[i].iTmpY * DELTA_Y + ORIGIN_Y);
                tBombs[i].sprExpl->Draw();					
                for(iTmp = tBombs[i].iLeft + 1; iTmp != tBombs[i].iTmpX; iTmp++)
                {
                    tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL+1);
                    tBombs[i].sprExpl->SetPosition((iTmp) * DELTA_X + ORIGIN_X , tBombs[i].iTmpY * DELTA_Y + ORIGIN_Y);
                    tBombs[i].sprExpl->Draw();
                }
            }

            // Right explosion
            if(tBombs[i].iRight != tBombs[i].iTmpX)
            {
                tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL + 3);
                tBombs[i].sprExpl->SetPosition((tBombs[i].iRight)*DELTA_X+ORIGIN_X,tBombs[i].iTmpY*DELTA_Y+ORIGIN_Y);
                tBombs[i].sprExpl->Draw();					
                for(iTmp = tBombs[i].iRight - 1; iTmp != tBombs[i].iTmpX; iTmp--)
                {
                    tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL + 1);
                    tBombs[i].sprExpl->SetPosition((iTmp) * DELTA_X + ORIGIN_X, tBombs[i].iTmpY * DELTA_Y + ORIGIN_Y);
                    tBombs[i].sprExpl->Draw();
                }
            }

            // Up explosion
            if(tBombs[i].iUp != tBombs[i].iTmpY)
            {
                tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL + 6);
                tBombs[i].sprExpl->SetPosition(tBombs[i].iTmpX * DELTA_X + ORIGIN_X, tBombs[i].iUp * DELTA_Y + ORIGIN_Y);
                tBombs[i].sprExpl->Draw();					
                for(iTmp = tBombs[i].iUp + 1; iTmp != tBombs[i].iTmpY; iTmp++)
                {
                    tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL + 4);
                    tBombs[i].sprExpl->SetPosition(tBombs[i].iTmpX * DELTA_X +ORIGIN_X, iTmp * DELTA_Y + ORIGIN_Y);
                    tBombs[i].sprExpl->Draw();
                }
            }

            // Down explosion
            if(tBombs[i].iDown != tBombs[i].iTmpY)
            {
                tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL + 5);
                tBombs[i].sprExpl->SetPosition(tBombs[i].iTmpX * DELTA_X + ORIGIN_X, tBombs[i].iDown * DELTA_Y + ORIGIN_Y);
                tBombs[i].sprExpl->Draw();					
                for(iTmp = tBombs[i].iDown - 1; iTmp != tBombs[i].iTmpY; iTmp--)
                {
                    tBombs[i].sprExpl->SelectAnim(SPRSTATE_NORMAL + 4);
                    tBombs[i].sprExpl->SetPosition(tBombs[i].iTmpX * DELTA_X + ORIGIN_X, iTmp * DELTA_Y + ORIGIN_Y);
                    tBombs[i].sprExpl->Draw();
                }
            }  

        }
    }
}