/*----------------------------------------------------------------------
TitanBomber - Roberto Prieto
Copyright (C) 2005-2023 MegaStorm Systems
contact@megastormsystems.com - http://www.megastormsystems.com

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

------------------------------------------------------------------------

object class

------------------------------------------------------------------------ */

#ifndef _OBJECTS
#define _OBJECTS

#include "CRM32Pro.h"
#include "gamefield.h"

#define DIR_NORMAL  1
#define DIR_DOWN    2
#define DIR_UP      4
#define DIR_LEFT    8
#define DIR_RIGHT  16

#define MAX_BOMBS   5
#define MAX_BOMBPOW 5
#define MAX_SPEED   7
#define MIN_SPEED   2
#define TIME_BOMB  5000
#define TIME_FIRE  800
#define TIME_INVUL 4000
#define TIME_DIR   150

#define START_LIFES   1
#define START_BOMBPOW 1
#define START_SPEED   4
#define START_BOMBS   1

#define ORIGIN_X  25
#define ORIGIN_Y  83
#define DELTA_X   50
#define DELTA_Y   45

#define PLAYER_EOMATCH  0
#define PLAYER_ALIVE    1 
#define PLAYER_KILLED   STATUS_RESERVED1 // 4
#define PLAYER_DEATH    STATUS_RESERVED2 // 8

#define KEY_UP     0
#define KEY_DOWN   1
#define KEY_LEFT   2
#define KEY_RIGHT  3
#define KEY_BOMB   4

// Clase de objetos base 
class cObjects
{
public:
    cObjects(char *,sPlayers *myself,cGameField *gfield);
    cObjects();
    ~cObjects();	
    Uint8 Execute();
    Uint8 Alive();
    void Debug();
    void Render();   
    int GetXr();
    int GetYr();
protected:	
    void TryMoveX(int iDir,int iSp=0);  // Try to move X axis on iDir at iSp
    void TryMoveY(int iDir,int iSp=0);  // Try to move Y axis on iDir at iSp
    int Abs2RelX(int);   // X coordinate conversion, pixel to map
    int Abs2RelY(int);   // Y coordinate conversion, pixel to map
    Uint8 BurnReach();   // Does reach us an explosion?

    // Objects properties
    cGameField *GameField; // Pointer to game field
    CRM32Pro_CSprite *sprObject;
    Uint8 iID;         // PLAYER_CPU,PLAYER_WATER, etc..
    int iXa,iYa;	   // Absolute position (on the screen without adjustment)
    int iXFix,iYFix;   // Position auto ajustment pending on its real size
    int iSpeed;        // Movement speed
    int iDirection;    // Current direction
    int iDirDelay;	   // Apply TIME_DIR delay before to change waiting animation
    Uint8 iLifes;      // Lifes 
    int iInvul;        // To control de Invul. time after a death
    SDL_mutex *mutPos; // Access to our position (when we are the server)
};

// Clase de objetos player derivada de la de objetos base
class cPlayer : public cObjects
{
public:
    cPlayer(char *,sPlayers *myself,cGameField *gfield);
    cPlayer();
    ~cPlayer();
    Uint8 Execute();
    void Control(int);       // Local/Remote control (send network events & use the reception queue)
    void AddPoints(int,int,int,int); // Add points 
    int GetPoints();         // Get points
    void SetCtrlKeys(int);
    void BombRender();		 // Draw bomb
    void InfoPlayer();       // Draw info(lifes,score..)
    void Survive();			 // Special player status used when the player survived to TIME_EOM
private:
    void ControlExecute(int,int,int);	// Execute the actions of Control
    void BombCreate();					// Try to create a new bomb
    int BombCreatePreValidate();		// Prevalidate a bomb creation request
    void BombExecute();					// Execute active bombs logic	
    void ItemCatch(Uint8,int,int);		// Allow to us to recolect items
    void ItemCatchQuery();				// Query to pick up an item. Used by network games
    void DeathCheckQuery(Uint8);		// Query a death check. Used by network games
    void SetInitPosition();				// Init position of the player		

    // Variables
    CRM32Pro_CSprite *sprExplMaster;
    CRM32Pro_CSprite *sprBombMaster;
    int tKeyboard[6];  // Array with keys to control the player
    int iDeathStatus;    // Death status of a player  
    Uint8 iWhoKilledMe;  // Store the killer
    int iPoints;       // Points
    char sName[8];     // Name of player
    Uint8 iRemote;     // 0: local player or 1: network player.
    Uint8 iID;         // ID of this player
    int iXBombFix,iYBombFix; // Auto ajuste de las bombas
    int iBombPow;      // Bombs power
    int iBombStore;    // Number of available bombs
    int iBombDelay;    // Delay in ms of explosion time	

    struct             // Bombs array
    {
        Sint32 iBangTime; 
        Sint32 iExplosionTime;
        Sint32 iOwner;
        CRM32Pro_CSprite *sprBomb;  // Bomb sprite cloned
        CRM32Pro_CSprite *sprExpl;  // Explosion sprite cloned
        Sint32 iTmpX,iTmpY;		    // Coordinates calculated on logic side and used on the blitting
        int iLeft,iRight,iUp,iDown; // Real explosion on each direction to be drawn
        int bDrawExplosion;         // Draw or not the explosion
        int iPower;					// Explosion power (capture from the player at the bomb creation time)
    } tBombs[MAX_BOMBS];

    // Estructura para el envio/recepcion de los eventos de los players
    sTBDN_Event TBDN_Event;	
    sTBDN_Event *pTBDN_Event;	
};

#endif
