/*----------------------------------------------------------------------
TitanBomber - Roberto Prieto
Copyright (C) 2005-2023 MegaStorm Systems
contact@megastormsystems.com - http://www.megastormsystems.com

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

------------------------------------------------------------------------

Resources

------------------------------------------------------------------------ */

#include "resource.h"
#include "gamefield.h"

#if defined(_LINUX) || defined(_MACOSX)
char *_itoa(int i,char *a,size_t len)
{
    sprintf(a,"%d",i % static_cast<int>(pow(10,len-1-(i<0))));
    return a;
}
#endif

// FontStore class
cFontStore *FontStore = NULL;

cFontStore::cFontStore()
{
}

int cFontStore::Load()
{
    Debug = new CRM32Pro_CFont;
    if(!Debug->Load(FONT_RESOURCE,"fDebug")) return 0;	

    Info = new CRM32Pro_CFont;
    if(!Info->Load(FONT_RESOURCE,"fInfo")) return 0;

    Score = new CRM32Pro_CFont;
    if(!Score->Load(FONT_RESOURCE,"fScore")) return 0;

    Air = new CRM32Pro_CFont;
    if(!Air->Load(FONT_RESOURCE,"fAir")) return 0;

    Earth = new CRM32Pro_CFont;
    if(!Earth->Load(FONT_RESOURCE,"fEarth")) return 0;

    Water = new CRM32Pro_CFont;
    if(!Water->Load(FONT_RESOURCE,"fWater")) return 0;

    Fire = new CRM32Pro_CFont;
    if(!Fire->Load(FONT_RESOURCE,"fFire")) return 0;

    return 1;
}

cFontStore::~cFontStore()
{
    if(Debug != NULL) { delete Debug; Debug = NULL; }
    if(Info != NULL) { delete Info; Info = NULL; }
    if(Score != NULL) { delete Score; Score = NULL; }
    if(Air != NULL) { delete Air; Air = NULL; }
    if(Earth != NULL) { delete Earth; Earth = NULL; }
    if(Water != NULL) { delete Water; Water = NULL; }
    if(Fire != NULL) { delete Fire; Fire = NULL; }
}

CRM32Pro_CFont *cFontStore::GetPlayerFont(Uint8 iID)
{
    switch(iID)
    {
    case PLAYER_WATER:
        return Water;
    case PLAYER_FIRE:
        return Fire;
    case PLAYER_EARTH:
        return Earth;
    case PLAYER_AIR:
        return Air;
    }
    return NULL;
}

// Sound & music resource
int hndS_thunder = 0, hndS_clickOK = 0, hndS_clickPS = 0, hndS_clickFail = 0, hndS_exit = 0;
int hndS_putBomb = 0, hndS_catchItem = 0, hndS_airExplosion = 0, hndS_waterExplosion = 0;
int hndS_earthExplosion = 0, hndS_fireExplosion = 0, hndS_death = 0;
int hndS_tieGame = 0, hndS_winGame = 0, hndS_abortGame = 0, hndS_startGame = 0;
int hndM_menu[3], hndM_air = 0, hndM_earth = 0, hndM_fire = 0, hndM_water = 0;

// LoadAudio() - Load audio resources
void LoadAudio()
{
    hndS_thunder = ISoundFX->SoundLoad(MUS_RESOURCE,"thunder");
    hndS_clickOK = ISoundFX->SoundLoad(MUS_RESOURCE,"clickOK");	
    hndS_clickPS = ISoundFX->SoundLoad(MUS_RESOURCE,"clickPS");	
    hndS_clickFail = ISoundFX->SoundLoad(MUS_RESOURCE,"clickFail");	
    hndS_exit = ISoundFX->SoundLoad(MUS_RESOURCE,"endplay");
    hndS_putBomb = ISoundFX->SoundLoad(MUS_RESOURCE,"put_bomb");
    hndS_catchItem = ISoundFX->SoundLoad(MUS_RESOURCE,"catch_item");
    hndS_airExplosion = ISoundFX->SoundLoad(MUS_RESOURCE,"air_explosion");
    hndS_earthExplosion = ISoundFX->SoundLoad(MUS_RESOURCE,"earth_explosion");
    hndS_waterExplosion = ISoundFX->SoundLoad(MUS_RESOURCE,"water_explosion");
    hndS_fireExplosion = ISoundFX->SoundLoad(MUS_RESOURCE,"fire_explosion");
    hndS_death = ISoundFX->SoundLoad(MUS_RESOURCE,"death_player");
    hndS_tieGame = ISoundFX->SoundLoad(MUS_RESOURCE,"tie_game");
    hndS_winGame = ISoundFX->SoundLoad(MUS_RESOURCE,"win_game");
    hndS_abortGame = ISoundFX->SoundLoad(MUS_RESOURCE,"abort_game");
    hndS_startGame = ISoundFX->SoundLoad(MUS_RESOURCE,"starting_game");

    hndM_menu[0] = ISoundFX->MusicLoad(MUS_RESOURCE,"menu1_music");
    hndM_menu[1] = ISoundFX->MusicLoad(MUS_RESOURCE,"menu2_music");
    hndM_menu[2] = ISoundFX->MusicLoad(MUS_RESOURCE,"menu3_music");
    hndM_air = ISoundFX->MusicLoad(MUS_RESOURCE,"air_music");
    hndM_earth = ISoundFX->MusicLoad(MUS_RESOURCE,"earth_music");
    hndM_fire = ISoundFX->MusicLoad(MUS_RESOURCE,"fire_music");
    hndM_water = ISoundFX->MusicLoad(MUS_RESOURCE,"water_music");
}

// FreeAudio() - Free audio resources
void FreeAudio()
{
    ISoundFX->SoundFree(hndS_thunder);
    ISoundFX->SoundFree(hndS_clickOK);
    ISoundFX->SoundFree(hndS_clickPS);
    ISoundFX->SoundFree(hndS_clickFail);
    ISoundFX->SoundFree(hndS_exit);
    ISoundFX->SoundFree(hndS_putBomb);
    ISoundFX->SoundFree(hndS_catchItem);
    ISoundFX->SoundFree(hndS_airExplosion);
    ISoundFX->SoundFree(hndS_earthExplosion);
    ISoundFX->SoundFree(hndS_waterExplosion);
    ISoundFX->SoundFree(hndS_fireExplosion);
    ISoundFX->SoundFree(hndS_death);
    ISoundFX->SoundFree(hndS_tieGame);
    ISoundFX->SoundFree(hndS_winGame);
    ISoundFX->SoundFree(hndS_abortGame);
    ISoundFX->SoundFree(hndS_startGame);

    ISoundFX->MusicFree(hndM_menu[0]);
    ISoundFX->MusicFree(hndM_menu[1]);
    ISoundFX->MusicFree(hndM_menu[2]);
    ISoundFX->MusicFree(hndM_air);
    ISoundFX->MusicFree(hndM_earth);
    ISoundFX->MusicFree(hndM_fire);
    ISoundFX->MusicFree(hndM_water);
}

// Graphics resource
SDL_Surface *sMenuBg,*sHallOfFameBg,*sAuthorBg;
Uint32 cCursor;
Uint32 bQuitID, bStartGameID, bJoinGameID, bHallOfFameID;
Uint32 bTimedID, bLeagueID, bRandomID;
Uint32 bAirNameID,bEarthNameID,bFireNameID,bWaterNameID;
Uint32 bAirSwitchID,bEarthSwitchID,bFireSwitchID,bWaterSwitchID;
Uint32 bIPID;
// LoadGraphics() - Load graphics resources
// return 1 if OK, 0 otherwise
int LoadGraphics()
{
    sMenuBg = IImage->Load(GFX_RESOURCE,"menubg");
    if(sMenuBg == NULL) return 0;
    sHallOfFameBg = IImage->Load(GFX_RESOURCE,"halloffamebg");
    if(sHallOfFameBg == NULL) return 0;
    sAuthorBg = IImage->Load(GFX_RESOURCE,"authorbg");
    if(sAuthorBg == NULL) return 0;

    cCursor = ICursor->Load(GFX_RESOURCE,"cursor");
    if(cCursor == -1) return 0;
    ICursor->Select(cCursor);
    FontStore = new cFontStore();
    if(!FontStore->Load()) return 0;

    bQuitID = IButton->Load(GFX_RESOURCE,"bExit");
    if(bQuitID == -1) return 0;
    bStartGameID = IButton->Load(GFX_RESOURCE,"bStartGame");
    if(bStartGameID == -1) return 0;
    bJoinGameID = IButton->Load(GFX_RESOURCE,"bJoinGame");
    if(bJoinGameID == -1) return 0;
    bTimedID = IButton->Load(GFX_RESOURCE,"bTimed");
    if(bTimedID == -1) return 0;
    bLeagueID = IButton->Load(GFX_RESOURCE,"bLeague");
    if(bLeagueID == -1) return 0;
    bRandomID = IButton->Load(GFX_RESOURCE,"bRandom");
    if(bRandomID == -1) return 0;
    bHallOfFameID = IButton->Load(GFX_RESOURCE,"bHallofFame");
    if(bHallOfFameID == -1) return 0;
    bIPID = IButton->Load(GFX_RESOURCE,"bIP");
    if(bIPID == -1) return 0;
    IButton->SetTextFont(bIPID,CRM32Pro.ConGetFont());
    //IButton->SetText(bIPID,"Server IP");
    IButton->SetText(bIPID,"10.66.196.238");
    bAirNameID = IButton->Load(GFX_RESOURCE,"bAirName");
    if(bAirNameID == -1) return 0;
    IButton->SetTextFont(bAirNameID,CRM32Pro.ConGetFont());
    IButton->SetText(bAirNameID,"Wind");
    bWaterNameID = IButton->Load(GFX_RESOURCE,"bWaterName");
    if(bWaterNameID == -1) return 0;
    IButton->SetTextFont(bWaterNameID,CRM32Pro.ConGetFont());
    IButton->SetText(bWaterNameID,"Rain");
    bFireNameID = IButton->Load(GFX_RESOURCE,"bFireName");
    if(bFireNameID == -1) return 0;
    IButton->SetTextFont(bFireNameID,CRM32Pro.ConGetFont());
    IButton->SetText(bFireNameID,"Sun");
    bEarthNameID = IButton->Load(GFX_RESOURCE,"bEarthName");
    if(bEarthNameID == -1) return 0;
    IButton->SetTextFont(bEarthNameID,CRM32Pro.ConGetFont());
    IButton->SetText(bEarthNameID,"Sand");
    bAirSwitchID = IButton->Load(GFX_RESOURCE,"bAirSwitch");
    if(bAirSwitchID == -1) return 0;
    bFireSwitchID = IButton->Load(GFX_RESOURCE,"bFireSwitch");
    if(bFireSwitchID == -1) return 0;
    bEarthSwitchID = IButton->Load(GFX_RESOURCE,"bEarthSwitch");
    if(bEarthSwitchID == -1) return 0;
    bWaterSwitchID = IButton->Load(GFX_RESOURCE,"bWaterSwitch");
    if(bWaterSwitchID == -1) return 0;

    return 1;
}

// FreeGraphics() - Free graphics resources
void FreeGraphics()
{
    delete FontStore; FontStore = NULL;
    CRM32Pro.FreeSurface(sMenuBg);
    CRM32Pro.FreeSurface(sHallOfFameBg);
    CRM32Pro.FreeSurface(sAuthorBg);
    IButton->RemoveAll();
    ICursor->Delete(cCursor);
}

// EnableButtons() - Set the button to the right state (depending on the values)
void EnableButtons(void *param1, void *param2)
{
    // Explicit cast to our right data type
    int *iGameType = (int*)param1;  

    IButton->SetState(bQuitID,B_NORMAL);
    IButton->SetState(bStartGameID,B_NORMAL);
    IButton->SetState(bJoinGameID,B_NORMAL);
    IButton->SetState(bTimedID,B_NORMAL);
    IButton->SetState(bLeagueID,B_NORMAL);
    IButton->SetState(bRandomID,B_NORMAL);
    IButton->SetState(bHallOfFameID,B_NORMAL);
    IButton->SetState(bIPID,B_NORMAL);
    IButton->SetState(bAirSwitchID,B_NORMAL);
    IButton->SetState(bFireSwitchID,B_NORMAL);
    IButton->SetState(bEarthSwitchID,B_NORMAL);
    IButton->SetState(bWaterSwitchID,B_NORMAL);

    if(IButton->GetValue(bEarthSwitchID) != 1) IButton->SetState(bEarthNameID,B_OFF);
    else IButton->SetState(bEarthNameID,B_NORMAL);

    if(IButton->GetValue(bFireSwitchID) != 1) IButton->SetState(bFireNameID,B_OFF);
    else IButton->SetState(bFireNameID,B_NORMAL);

    if(IButton->GetValue(bWaterSwitchID) != 1) IButton->SetState(bWaterNameID,B_OFF);
    else IButton->SetState(bWaterNameID,B_NORMAL);

    if(IButton->GetValue(bAirSwitchID) != 1)IButton->SetState(bAirNameID,B_OFF);
    else IButton->SetState(bAirNameID,B_NORMAL);

    switch(*iGameType)
    {
    case GAMETYPE_LEAGUE:
        IButton->SetState(bLeagueID,B_PRESS);
        break;
    case GAMETYPE_TIMED:
        IButton->SetState(bTimedID,B_PRESS);
        break;
    case GAMETYPE_RANDOM:
        IButton->SetState(bRandomID,B_PRESS);
        break;
    }
}

// DisableButtons() - Set all the buttons to off status
void DisableButtons()
{
    IButton->SetState(bQuitID,B_OFF);
    IButton->SetState(bStartGameID,B_OFF);
    IButton->SetState(bJoinGameID,B_OFF);
    IButton->SetState(bTimedID,B_OFF);
    IButton->SetState(bLeagueID,B_OFF);
    IButton->SetState(bRandomID,B_OFF);
    IButton->SetState(bHallOfFameID,B_OFF);
    IButton->SetState(bIPID,B_OFF);
    IButton->SetState(bEarthNameID,B_OFF);
    IButton->SetState(bFireNameID,B_OFF);
    IButton->SetState(bWaterNameID,B_OFF);
    IButton->SetState(bAirNameID,B_OFF);
    IButton->SetState(bAirSwitchID,B_OFF);
    IButton->SetState(bFireSwitchID,B_OFF);
    IButton->SetState(bEarthSwitchID,B_OFF);
    IButton->SetState(bWaterSwitchID,B_OFF);
}