/*----------------------------------------------------------------------
TitanBomber - Roberto Prieto
Copyright (C) 2005-2023 MegaStorm Systems
contact@megastormsystems.com - http://www.megastormsystems.com

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

------------------------------------------------------------------------

TitanBomber

------------------------------------------------------------------------ */

// ---Defines---
#define GAME_VERSION "TitanBomber v0.99 Beta 014"

//int s=0; int f=0;

// ---Includes---
#include <stdlib.h>
#include <stdio.h>
#include <time.h>         
#include <string.h>
#include "CRM32Pro.h" 
#include "titanbomber.h"
#include "gamefield.h"
#include "resource.h"
#include "hof.h"

// ---Global vars---
int iPlayersLocal,iPlayersRemote;
int iMenuScreen;

// ---Main function---
int main( int argc, char* argv[] )
{  
    Uint32 bID;
    int done_menu,done_match,i,j,k;
    int iGamePlayedTime;
    int tRandomMap[NUMBER_OF_MAP];
    SDL_Event myEvent;    
    int iGameType,iClientGameType;
    cGameField *GameField = NULL;
    sPlayers tPlayers[4];

    // -Log system-
    ILogSystem.Init("titanbomber.log",LOG_FILE | LOG_CONSOLE,LOG_NORMAL,GAME_VERSION);

    // -CRM32Pro init- 
    if(CRM32Pro.Init(SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_AUDIO)<0) 
    {
        ILogSystem.Msg(LOG_NORMAL," [Game Init Error] - Couldn't initialize CRM32Pro: %s\n",SDL_GetError());
        return 1; 
    }    

    // -Desired config-
    if(!CRM32Pro.LoadConfig("setup.dpf","setup.xml"))
    {
        ILogSystem.Msg(LOG_NORMAL," [Game Init Error] - Couldn't load 'setup.xml'. Please, run 'Setup.exe'\n");
        CRM32Pro.Quit();
        return 1;
    }
    CRM32Pro.Config.Title = GAME_VERSION;
    CRM32Pro.Config.Icon = IImage->Load(GFX_RESOURCE,"icono");

    // -Graphics system initialize-
    if(!CRM32Pro.SetVideoMode()) 
    {
        ILogSystem.Msg(LOG_NORMAL," [Game Init Error] - Couldn't set video mode 800x600: %s\n",SDL_GetError());
        CRM32Pro.Quit();
        return 1;
    }
    ICursor->Init();

    // -Console initialization-
    CRM32Pro.ConTitle(GAME_VERSION);
    CRM32Pro.ConEnable();
    CRM32Pro.ConResize(285,125);
    CRM32Pro.ConMove(262,480);	
    CRM32Pro.ConFeatures(CONFEATURE_DISABLE_KEY | CONFEATURE_OUTPUT_ONLY);

    // -Init sound system-
    done_menu = ISoundFX->Init(); // It uses the config of CRM32Pro.Config
    if(done_menu == -1) ILogSystem.Msg(LOG_NORMAL," [Game Init Warning] - Couldn't init sound system: %s\n",SDL_GetError());	
    else if(done_menu == 1) 
    {		
        ISoundFX->ChannelVolume(-1,CRM32Pro.Config.AudioSoundVol);
        ISoundFX->MusicVolume(CRM32Pro.Config.AudioMusicVol);
        LoadAudio();
    }

    // -MegaStorm Systems presentation-        
    sMenuBg = IImage->Load(GFX_RESOURCE, "logotipoMS");
    SDL_FillRect(CRM32Pro.screen, NULL, SDL_MapRGB(CRM32Pro.screen->format, 0, 0, 0));
    ISoundFX->SoundPlay(-1, hndS_thunder, 0);
    IScreenFX->FadeToImage(sMenuBg, 5000);
    IScreenFX->FadeToColor(0, 0, 0, 5000);
    SDL_FreeSurface(sMenuBg);

    // -Load graphics-
    if(!LoadGraphics())
    {
        FreeAudio();
        FreeGraphics();   
        ILogSystem.Msg(LOG_NORMAL," [Game Init Error] - Couldn't load graphics resources\n");
        CRM32Pro.Quit();
        return 1;
    }     

    // -Init time system, reset players and prepare all subsystem-
    iMenuScreen = MENUSCREEN_MAIN;
    ITimeSystem->Init();
    ITimeSystem->SetRate(0,20);  
    ResetPlayers(&tPlayers[0]);
    CRM32Pro.RandSeed(SDL_GetTicks());
    iGameType = GAMETYPE_LEAGUE;
    HOF = new cHOF;
    GameField = new cGameField;

    // ---Menu logic---
    done_menu = TB_ENTRY;
    while(done_menu != TB_QUIT)
    {
        bID = IButton->Check();

        // 1.First execution or after a game match: 
        //  - initial fade, music play, show cursor
        //  - set our render callback
        if(done_menu == TB_ENTRY)
        {
            ISoundFX->MusicPlay(hndM_menu[CRM32Pro.Rand() % 3], 100);  
            IScreenFX->FadeToImage(sMenuBg,500);
            ICursor->Show();
            ICursor->Invalidate();
            EnableButtons(&iGameType);
            CRM32Pro.SetRenderCallback(RenderInMenu);
            CRM32Pro.CleanUp();
            IButton->SetLayerVisibility(0,1);
            bID = -1;
            done_menu = TB_LOOP;
            CRM32Pro.RenderNeeded();
        }	

        // 2.Buttons management. The bID is retrieved by our render callback.   
        // 2.1.Quit to OS
        if(bID == bQuitID)
        {
            ISoundFX->SoundPlay(-1,hndS_exit,0);			
            done_menu = TB_QUIT;
        }
        // 2.2.Start game
        else if(bID == bStartGameID)
        {
            // Check if we can startup the game: minimum 2 players (local+local or local+remote)
            AssignPlayers(&tPlayers[0]); 
            if(iPlayersLocal + iPlayersRemote > 1)
            {
                ISoundFX->SoundPlay(-1,hndS_clickOK,0);

                // Prepare a random map list to run	
                for(k = 0; k < NUMBER_OF_MAP; k++) tRandomMap[k] = k;
                for(k = NUMBER_OF_MAP; k > 1; k--)
                {
                    i = CRM32Pro.Rand() % k;
                    j = tRandomMap[i];
                    tRandomMap[i] = tRandomMap[k - 1];
                    tRandomMap[k - 1] = j;
                }
                i = 0; // i to navigate through the map table
                iGamePlayedTime = 0; // to save the played time(for timed matches)

                done_match = GAME_GO;
                while(done_match == GAME_GO)
                {
                    // Game type (for timed matches) logic
                    if(iGameType == GAMETYPE_TIMED) j = TIMED_MATCH - iGamePlayedTime;
                    else j = 0;

                    // Single player mode
                    if(iPlayersRemote == 0) done_match = GameField->Init(GAMEMODE_LOCAL,GameField->GetMapName(tRandomMap[i]),tPlayers,iGameType,j);
                    // Multiplayer mode (server)
                    else
                    {
                        DisableButtons();
                        done_match = GameField->Init(GAMEMODE_SERVER,GameField->GetMapName(tRandomMap[i]),tPlayers,iGameType,j);
                        //INetwork->SetSimDelay(50);
                    }

                    // Executes the selected map
                    if(done_match == GAME_GO) 
                    {						
                        done_match = GameField->Execute();	 
                        if(iGameType == GAMETYPE_TIMED) iGamePlayedTime = iGamePlayedTime + GameField->GetPlayedTime();
                        ILogSystem.Msg(LOG_NORMAL,"\n [Match over] - P1 %d - P2 %d - P3 %d - P4 %d (%s - %d)\n",tPlayers[0].points,tPlayers[1].points,tPlayers[2].points,tPlayers[3].points,GameField->GetGameTypeName(),done_match);
                        done_menu = TB_ENTRY;
                    } 	   
                    // Close the game field (it requires a new Init() before executing another map)
                    GameField->Close();

                    // Match cycle for league and timed game types
                    switch(iGameType)
                    {
                        // League, a whole cycle through all available maps
                    case GAMETYPE_LEAGUE:
                        if(i < (NUMBER_OF_MAP - 1)) 
                        {
                            i++;
                            // Detect the last match and sends the right message on network games to the clients
                            if(i == (NUMBER_OF_MAP - 1)) iGameType = GAMETYPE_LEAGUE_LAST;
                        }
                        break;
                        // Timed, play during TIMED_MATCH seconds
                    case GAMETYPE_TIMED:
                        if(i < (NUMBER_OF_MAP - 1)) i++;
                        else i = 0;
                        if((TIMED_MATCH - iGamePlayedTime) > 0) 
                            ILogSystem.Msg(LOG_NORMAL," [Game Info] - Timed game with %dseg left\n",TIMED_MATCH - iGamePlayedTime);
                        break;
                    }
                }
                // End of game, save the points to HOF, reset the players table and enable the buttons
                if(iGameType == GAMETYPE_LEAGUE_LAST) iGameType = GAMETYPE_LEAGUE; // GAMETYPE_LEAGUE_LAST is the same as GAMETYPE_LEAGUE but used to detect the last match
                HOF->Store(tPlayers,iGameType);
                ResetPlayers(&tPlayers[0]);	
                EnableButtons(&iGameType);	
            }
            else ISoundFX->SoundPlay(-1,hndS_clickFail,0);
        }
        // 2.2.Join game
        else if(bID == bJoinGameID)
        {			
            // Check if we can join to a game: only allow one local player(right name) and check the IP of the server	
            AssignPlayers(&tPlayers[0]); 
            if((strlen(IButton->GetText(bIPID)) > 6) && (iPlayersLocal == 1)) // ToDO: could we use a better way to detect an ip address?
            {
                ISoundFX->SoundPlay(-1,hndS_clickOK,0);
                done_match = GAME_GO;
                while(done_match == GAME_GO)
                {
                    DisableButtons();					
                    done_match = GameField->Init(GAMEMODE_CLIENT,IButton->GetText(bIPID),tPlayers,0,0);
                    //INetwork->SetSimDelay(50);

                    if(done_match == GAME_GO) 
                    {	
                        done_match = GameField->Execute();	 
                        ILogSystem.Msg(LOG_NORMAL,"\n [Match over] - P1 %d - P2 %d - P3 %d - P4 %d (%s - %d)\n",tPlayers[0].points,tPlayers[1].points,tPlayers[2].points,tPlayers[3].points,GameField->GetGameTypeName(),done_match);
                        done_menu = TB_ENTRY;
                    }	
                    iClientGameType = GameField->GetGameType(); // Get the gametype (it was sent by the server)
                    GameField->Close();
                    // The end of the game is sent by the server					
                }
                // End of game, save the points to HOF, reset the players table and enable the buttons				
                if(iClientGameType == GAMETYPE_LEAGUE_LAST) iClientGameType = GAMETYPE_LEAGUE; // GAMETYPE_LEAGUE_LAST is the same as GAMETYPE_LEAGUE but used to detect the last match
                HOF->Store(tPlayers,iClientGameType);   // iGameType is not used when we are a client
                ResetPlayers(&tPlayers[0]);	
                EnableButtons(&iClientGameType);		// iGameType is not used when we are a client
            }
            else 
            {
                ISoundFX->SoundPlay(-1,hndS_clickFail,0);
                ResetPlayers(&tPlayers[0]);
            }
        }
        // 2.3.Change game type to random (1 match)
        else if(bID == bRandomID)
        {
            ISoundFX->SoundPlay(-1,hndS_clickOK,0);
            iGameType = GAMETYPE_RANDOM;
            EnableButtons(&iGameType);
        }
        // 2.4.Change game type to timed (5 minutes)
        else if(bID == bTimedID)
        {
            ISoundFX->SoundPlay(-1,hndS_clickOK,0);			
            iGameType = GAMETYPE_TIMED;
            EnableButtons(&iGameType);
        }
        // 2.5.Change game type to league
        else if(bID == bLeagueID)
        {
            ISoundFX->SoundPlay(-1,hndS_clickOK,0);
            iGameType = GAMETYPE_LEAGUE;
            EnableButtons(&iGameType);
        }      
        // 2.5.Hall of Fame
        else if(bID == bHallOfFameID)
        {
            ISoundFX->SoundPlay(-1,hndS_clickOK,0);
            IScreenFX->FadeToImage(sHallOfFameBg,500);	
            IButton->SetLayerVisibility(0,0);
            iMenuScreen = MENUSCREEN_HOF;
        }
        // 2.6.Player switches
        else if((bID == bAirSwitchID) || (bID == bFireSwitchID) || (bID == bWaterSwitchID) || (bID == bEarthSwitchID))
        {
            ISoundFX->SoundPlay(-1,hndS_clickPS,0);
            EnableButtons(&iGameType);
        }

        // 3.Main update: blit to screen, update time system, etc.
        while(CRM32Pro.Update(&myEvent))
        {
            // Manage some events: window closing and change from Hall of Fame to Menu
            switch(myEvent.type)
            {
            case SDL_QUIT:
                done_menu = TB_QUIT;
                break;
            case SDL_MOUSEBUTTONUP:
                // any pressed mouse button on the Hall Of Fame screen, will return to the menu
                if(iMenuScreen == MENUSCREEN_HOF)
                {				
                    ISoundFX->SoundPlay(-1,hndS_clickOK,0);
                    iMenuScreen = MENUSCREEN_MAIN;
                    done_menu = TB_ENTRY;
                }
                break;
            case SDL_KEYDOWN:
                // any pressed key on the Hall Of Fame screen, will return to the menu
                if(iMenuScreen == MENUSCREEN_HOF)
                {
                    iMenuScreen = MENUSCREEN_MAIN;
                    done_menu = TB_ENTRY;
                }
                else if(myEvent.key.keysym.sym == SDLK_ESCAPE) done_menu = TB_QUIT;
                break;
            }
        }
    }

    // -Exiting...-
    delete HOF;
    if(GameField != NULL) delete GameField;
    ICursor->Hide(); 	
    ISoundFX->MusicFadeOut(1000);
    IScreenFX->FadeToColor(0,0,0,1000);
    ISoundFX->MusicStop();
    FreeAudio();
    FreeGraphics();   
    ILogSystem.Msg(LOG_NORMAL,"\n [Game Info] - End of TitanBomber game\n");
    CRM32Pro.AudioInfo();
    CRM32Pro.VideoInfo();
    CRM32Pro.Quit();
    return 0;
}

// Callback function called by CRM32Pro.Update() fulfilling with Rendering Frame Rate
// Used in the menu screen. Not in the game.
void RenderInMenu(int bLogicUpdate)
{
    if(!CRM32Pro.IsRenderNeeded()) return;
    {
        SDL_Rect rRect;

        // Main menu screen
        if(iMenuScreen == MENUSCREEN_MAIN) 
        {
            CRM32Pro.Blit(sMenuBg,NULL,CRM32Pro.screen,NULL);
        }
        // Hall of fame screen
        else
        {
            CRM32Pro.Blit(sHallOfFameBg,NULL,CRM32Pro.screen,NULL);
            HOF->Render();
        }
        // Common gfx (author and buttons)
        rRect.x = (CRM32Pro.screen->w - sAuthorBg->w) /2; rRect.y = CRM32Pro.screen->h - sAuthorBg->h; rRect.w = sAuthorBg->w; rRect.h = sAuthorBg->h;
        CRM32Pro.Blit(sAuthorBg,NULL,CRM32Pro.screen,&rRect);
        IButton->Draw();
    } 
}

// Check for duplicated names: if any, return 0. Otherwise, return 1
int CheckName(sPlayers *tP,char *name)
{
    int i;
    for(i = 0; i < 4; i++)
    {
        if(!strncmp(name,tP[i].name,4)) return 0;
    }
    return 1;
}

// Assign Players to theirs struct
// It also check for valid local players (with text)
void AssignPlayers(sPlayers *tP)
{
    int i;
    char *sTmp;

    iPlayersLocal = iPlayersRemote = 0;

    // Water player check
    i = IButton->GetValue(bWaterSwitchID);
    switch(i)
    {
        // Remote player
    case 0: 
        tP[0].id = PLAYER_WATER;
        tP[0].remote = 1;
        iPlayersRemote++;
        break;
        // Local player
    case 1:
        sTmp = IButton->GetText(bWaterNameID);
        if(strlen(sTmp) > 0)
        {
            strncpy(tP[0].name,sTmp,4);
            if(strlen(sTmp) > 3) tP[0].name[4]='\0';
            tP[0].id = PLAYER_WATER;
            tP[0].remote = 0;
            iPlayersLocal++;
        }
        break;
        // Nothing to do, this player does not play this game
    case 2:			
        break;
    }

    // Fire player check
    i = IButton->GetValue(bFireSwitchID);
    switch(i)
    {
        // Remote player
    case 0: 			
        tP[1].id = PLAYER_FIRE;
        tP[1].remote = 1;
        iPlayersRemote++;
        break;
        // Local player
    case 1:
        sTmp = IButton->GetText(bFireNameID);
        // Check player name: it must be unique
        if((strlen(sTmp) > 0) && (CheckName(tP,sTmp)))
        {
            strncpy(tP[1].name,sTmp,4);
            if(strlen(sTmp) > 3) tP[1].name[4]='\0';
            tP[1].id = PLAYER_FIRE;
            tP[1].remote = 0;
            iPlayersLocal++;
        }
        break;
        // Nothing to do, this player does not play this game
    case 2:			
        break;
    }

    // Earth player check
    i = IButton->GetValue(bEarthSwitchID);
    switch(i)
    {
        // Remote player
    case 0: 
        tP[2].id = PLAYER_EARTH;
        tP[2].remote = 1;
        iPlayersRemote++;
        break;
        // Local player
    case 1:
        sTmp = IButton->GetText(bEarthNameID);
        // Check player name: it must be unique
        if((strlen(sTmp) > 0) && (CheckName(tP,sTmp)))
        {
            strncpy(tP[2].name,sTmp,4);
            if(strlen(sTmp) > 3) tP[2].name[4]='\0';
            tP[2].id = PLAYER_EARTH;
            tP[2].remote = 0;
            iPlayersLocal++;
        }
        break;
        // Nothing to do, this player does not play this game
    case 2:			
        break;
    }

    // Air player check
    i = IButton->GetValue(bAirSwitchID);
    switch(i)
    {
        // Remote player
    case 0: 
        tP[3].id = PLAYER_AIR;			
        tP[3].remote = 1;
        iPlayersRemote++;
        break;
        // Local player
    case 1:
        sTmp = IButton->GetText(bAirNameID);
        // Check player name: it must be unique
        if((strlen(sTmp) > 0) && (CheckName(tP,sTmp)))
        {
            strncpy(tP[3].name,sTmp,4);
            if(strlen(sTmp) > 3) tP[3].name[4]='\0';
            tP[3].id = PLAYER_AIR;
            tP[3].remote = 0;
            iPlayersLocal++;
        }
        break;
        // Nothing to do, this player does not play this game
    case 2:			
        break;
    }
}

// Reset Players struct
void ResetPlayers(sPlayers *tP)
{
    int i;
    iPlayersLocal = iPlayersRemote = 0;

    for(i = 0; i < 4; i++)
    {
        tP[i].player = NULL;
        strcpy(tP[i].name,"");
        tP[i].points = 0;
        tP[i].bonus = 0;
        tP[i].id = 0;
        tP[i].remote = 0;
        tP[i].iDelay = 0;
    }
}

