/*----------------------------------------------------------------------
TitanBomber - Roberto Prieto
Copyright (C) 2005-2023 MegaStorm Systems
contact@megastormsystems.com - http://www.megastormsystems.com

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

------------------------------------------------------------------------

TitanBomber

Bomberman like-game with 4 different game fields (one for each Titan):
- 2D environment
- 800x600 with 32bits
- Up to 4 players local or network (running on any supported platform)
- Three different game modes: league, timed and random
- Highscore table for each game mode

Changelog:
- Check History.txt

------------------------------------------------------------------------ */

#ifndef _TITANBOMBER
#define _TITANBOMBER

// ---Defines---
#define GFX_RESOURCE "game.dpf"
#define MUS_RESOURCE "game.dpf"

#define TB_ENTRY    0
#define TB_LOOP     1
#define TB_QUIT     2
#define TB_ENDMATCH 3

#define GAMETYPE_LEAGUE_LAST 0
#define GAMETYPE_LEAGUE	1
#define GAMETYPE_TIMED  2
#define GAMETYPE_RANDOM 3

#define MENUSCREEN_MAIN 0
#define MENUSCREEN_HOF  1

#define TIMED_MATCH 60 // 5 minutes(in seconds) (5*60)

//extern int s,f;

// ---Players struct---
struct sPlayers
{
    char name[8];   // Name of player,if NULL, there isnt player.
    class cPlayer *player; // Cross-reference to avoid cyclic header dependency
    int points;    // Store player points across all the maps.
    Uint8 bonus;   // Add a life to player who won previous match
    Uint8 remote;  // 0: local player or 1: network player.
    Uint8 id;      // PLAYER_WATER,etc..
    Uint32 iDelay; // Delay to start the match
};

// ---Prototypes---
int CheckName(sPlayers *,char *);
void ResetPlayers(sPlayers *);
void AssignPlayers(sPlayers *);
void RenderInMenu(int bLogicUpdate);

// ---Network packages: defines and struct---
#define TBDN_NULL         0   // Null DN
#define TBDN_GAMEEND      2   // Set the end of the match
#define TBDN_GAMEABORT    3   // Set the abort of the game
#define TBDN_SENDPLAYERS  6   // Package with players struct. Sent by the server once it is validated
#define TBDN_SENDMAP      7   // Package with the map struct
#define TBDN_STARTMAP	  8   // Send the start game signal including the delay
#define TBDN_GAMETYPE     9   // Package with the game type 
#define TBDN_KILLPLAYER  10   // Remove this player from the game
#define TBDN_NEWPLAYER   11   // Sent by a new player to try to get a gaming slot
#define TBDN_READYPLAYER 12   // Sent by the player when it is ready
#define TBDN_P1EVENT     16   // Events for player 1
#define TBDN_P2EVENT     32   // Events for player 2
#define TBDN_P3EVENT     64   // Events for player 3
#define TBDN_P4EVENT    128   // Events for player 4

struct sTBDN        // Para envio/recepcion del mapa/players, son 512bytes
{
    Uint8 type;
    union 
    {
        char data[511];	
        sPlayers player[4];
    };
};	

// Event DN. 
struct sTBDN_Event  
{
    Uint8 iCtrlID;    // Player ID or TBDN_GAMENEXT or TBDN_GAMEEND
    struct sEvent
    {
        Uint8 move_x; // Set movement on X axis
        Uint8 move_y; // Set movement on Y axis
        Uint8 Xr;     // X relative position (on the map) of the player. With points != 0, it is its X position not the player
        Uint8 Yr;     // Y relative position (on the map). With points != 0, it is its X position not the player
        Uint8 bomb;   // Ask for bomb creation
        Uint8 item;	  // Ask for getting an item
        Uint8 death;  // Initiate a death event 
        Sint16 points;// Points event
    } event;
};	

extern int iPlayersLocal,iPlayersRemote;

#endif
